@@ -93,7 +93,8 @@ type Sandbox struct {
 	DefaultChattr   string   `json:"default_chattr"`
 	DefaultChroot   string   `json:"default_chroot"`
 	DefaultUtime    string   `json:"default_utime"`
-	DefaultMkdev    string   `json:"default_mkdev"`
+	DefaultMkbdev   string   `json:"default_mkbdev"`
+	DefaultMkcdev   string   `json:"default_mkcdev"`
 	DefaultMkfifo   string   `json:"default_mkfifo"`
 	DefaultMktemp   string   `json:"default_mktemp"`
 
@@ -916,35 +917,67 @@ func EnabledUtime() bool {
 	return bool(result)
 }
 
-// EnableMkdev enables mkdev sandboxing.
+// EnableMkbdev enables mkbdev sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
 // on failure. The error is of type syscall.Errno.
-func EnableMkdev() error {
-	result := C.syd_enable_mkdev()
+func EnableMkbdev() error {
+	result := C.syd_enable_mkbdev()
 	if result != 0 {
 		return syscall.Errno(-result)
 	}
 	return nil
 }
 
-// DisableMkdev disables mkdev sandboxing.
+// DisableMkbdev disables mkbdev sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
 // on failure. The error is of type syscall.Errno.
-func DisableMkdev() error {
-	result := C.syd_disable_mkdev()
+func DisableMkbdev() error {
+	result := C.syd_disable_mkbdev()
 	if result != 0 {
 		return syscall.Errno(-result)
 	}
 	return nil
 }
 
-// EnabledMkdev checks if mkdev sandboxing is enabled in the syd environment.
+// EnabledMkbdev checks if mkbdev sandboxing is enabled in the syd environment.
 //
-// It returns true if mkdev sandboxing is enabled, and false otherwise.
-func EnabledMkdev() bool {
-	result := C.syd_enabled_mkdev()
+// It returns true if mkbdev sandboxing is enabled, and false otherwise.
+func EnabledMkbdev() bool {
+	result := C.syd_enabled_mkbdev()
+	return bool(result)
+}
+
+// EnableMkcdev enables mkcdev sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func EnableMkcdev() error {
+	result := C.syd_enable_mkcdev()
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// DisableMkcdev disables mkcdev sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func DisableMkcdev() error {
+	result := C.syd_disable_mkcdev()
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// EnabledMkcdev checks if mkcdev sandboxing is enabled in the syd environment.
+//
+// It returns true if mkcdev sandboxing is enabled, and false otherwise.
+func EnabledMkcdev() bool {
+	result := C.syd_enabled_mkcdev()
 	return bool(result)
 }
 
@@ -1437,12 +1470,24 @@ func DefaultUtime(action Action) error {
 	return nil
 }
 
-// Set default action for Mkdev sandboxing.
+// Set default action for Mkbdev sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
 // on failure. The error is of type syscall.Errno.
-func DefaultMkdev(action Action) error {
-	result := C.syd_default_mkdev(C.action_t(action))
+func DefaultMkbdev(action Action) error {
+	result := C.syd_default_mkbdev(C.action_t(action))
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// Set default action for Mkcdev sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func DefaultMkcdev(action Action) error {
+	result := C.syd_default_mkcdev(C.action_t(action))
 	if result != 0 {
 		return syscall.Errno(-result)
 	}
@@ -2529,48 +2574,96 @@ func UtimeRem(action Action, glob string) error {
 	return nil
 }
 
-// MkdevAdd adds the specified glob pattern to the given actionlist of
-// Mkdev sandboxing.
+// MkbdevAdd adds the specified glob pattern to the given actionlist of
+// Mkbdev sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
 // on failure. The error is of type syscall.Errno.
-func MkdevAdd(action Action, glob string) error {
+func MkbdevAdd(action Action, glob string) error {
 	cGlob := C.CString(glob)
 	defer C.free(unsafe.Pointer(cGlob))
 
-	result := C.syd_mkdev_add(C.action_t(action), cGlob)
+	result := C.syd_mkbdev_add(C.action_t(action), cGlob)
 	if result != 0 {
 		return syscall.Errno(-result)
 	}
 	return nil
 }
 
-// MkdevDel removes the first instance from the end of the given
-// actionlist of mkdev sandboxing.
+// MkbdevDel removes the first instance from the end of the given
+// actionlist of mkbdev sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
 // on failure. The error is of type syscall.Errno.
-func MkdevDel(action Action, glob string) error {
+func MkbdevDel(action Action, glob string) error {
 	cGlob := C.CString(glob)
 	defer C.free(unsafe.Pointer(cGlob))
 
-	result := C.syd_mkdev_del(C.action_t(action), cGlob)
+	result := C.syd_mkbdev_del(C.action_t(action), cGlob)
 	if result != 0 {
 		return syscall.Errno(-result)
 	}
 	return nil
 }
 
-// MkdevRem removes all matching patterns from the given actionlist of
-// Mkdev sandboxing.
+// MkbdevRem removes all matching patterns from the given actionlist of
+// Mkbdev sandboxing.
 //
 // Returns nil on success, and an error corresponding to the negated errno
 // on failure. The error is of type syscall.Errno.
-func MkdevRem(action Action, glob string) error {
+func MkbdevRem(action Action, glob string) error {
 	cGlob := C.CString(glob)
 	defer C.free(unsafe.Pointer(cGlob))
 
-	result := C.syd_mkdev_rem(C.action_t(action), cGlob)
+	result := C.syd_mkbdev_rem(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// MkcdevAdd adds the specified glob pattern to the given actionlist of
+// Mkcdev sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func MkcdevAdd(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_mkcdev_add(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// MkcdevDel removes the first instance from the end of the given
+// actionlist of mkcdev sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func MkcdevDel(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_mkcdev_del(C.action_t(action), cGlob)
+	if result != 0 {
+		return syscall.Errno(-result)
+	}
+	return nil
+}
+
+// MkcdevRem removes all matching patterns from the given actionlist of
+// Mkcdev sandboxing.
+//
+// Returns nil on success, and an error corresponding to the negated errno
+// on failure. The error is of type syscall.Errno.
+func MkcdevRem(action Action, glob string) error {
+	cGlob := C.CString(glob)
+	defer C.free(unsafe.Pointer(cGlob))
+
+	result := C.syd_mkcdev_rem(C.action_t(action), cGlob)
 	if result != 0 {
 		return syscall.Errno(-result)
 	}
