@@ -21,7 +21,7 @@ use nix::{
 };
 use once_cell::sync::Lazy;
 
-use crate::{fs::fillrandom, kernel::rename::RenameFlags, path::XPath};
+use crate::{compat::RenameFlags, fs::fillrandom, path::XPath};
 
 /// A platform-sized secure cookie
 ///
@@ -131,6 +131,8 @@ pub(crate) static UNLINKAT_COOKIE_ARG4: Lazy<Cookie> =
 #[allow(clippy::disallowed_methods)]
 pub(crate) static UNLINKAT_COOKIE_ARG5: Lazy<Cookie> =
     Lazy::new(|| getcookie().expect("getcookie"));
+#[allow(clippy::disallowed_methods)]
+pub(crate) static LINKAT_COOKIE_ARG5: Lazy<Cookie> = Lazy::new(|| getcookie().expect("getcookie"));
 
 /// These are used in `syd::fs::seccomp_notify_addfd`.
 #[allow(clippy::disallowed_methods)]
@@ -367,9 +369,11 @@ pub(crate) fn safe_truncate(path: &XPath, len: libc::off_t) -> Result<(), Errno>
 pub(crate) fn safe_truncate64(path: &XPath, len: libc::off64_t) -> Result<(), Errno> {
     #[cfg(not(any(
         target_pointer_width = "64",
+        all(target_arch = "x86_64", target_pointer_width = "32"),
         target_arch = "x86",
         target_arch = "arm",
         target_arch = "powerpc",
+        target_arch = "m68k",
         target_arch = "mips",
         target_arch = "mips32r6",
     )))]
@@ -377,12 +381,15 @@ pub(crate) fn safe_truncate64(path: &XPath, len: libc::off64_t) -> Result<(), Er
         compile_error!("BUG: safe_truncate64 is not implemented for this architecture!");
     }
 
-    #[cfg(target_pointer_width = "64")]
+    #[cfg(any(
+        target_pointer_width = "64",
+        all(target_arch = "x86_64", target_pointer_width = "32"),
+    ))]
     {
         safe_truncate(path, len)
     }
 
-    #[cfg(target_arch = "x86")]
+    #[cfg(any(target_arch = "m68k", target_arch = "x86",))]
     {
         let sys_truncate64 = SYS_TRUNCATE64.ok_or(Errno::ENOSYS)?;
 
@@ -459,9 +466,11 @@ pub(crate) fn safe_ftruncate<Fd: AsFd>(fd: Fd, len: libc::off_t) -> Result<(), E
 pub(crate) fn safe_ftruncate64<Fd: AsFd>(fd: Fd, len: libc::off64_t) -> Result<(), Errno> {
     #[cfg(not(any(
         target_pointer_width = "64",
+        all(target_arch = "x86_64", target_pointer_width = "32"),
         target_arch = "x86",
         target_arch = "arm",
         target_arch = "powerpc",
+        target_arch = "m68k",
         target_arch = "mips",
         target_arch = "mips32r6",
     )))]
@@ -469,12 +478,15 @@ pub(crate) fn safe_ftruncate64<Fd: AsFd>(fd: Fd, len: libc::off64_t) -> Result<(
         compile_error!("BUG: safe_ftruncate64 is not implemented for this architecture!");
     }
 
-    #[cfg(target_pointer_width = "64")]
+    #[cfg(any(
+        target_pointer_width = "64",
+        all(target_arch = "x86_64", target_pointer_width = "32"),
+    ))]
     {
         safe_ftruncate(fd, len)
     }
 
-    #[cfg(target_arch = "x86")]
+    #[cfg(any(target_arch = "m68k", target_arch = "x86",))]
     {
         let sys_ftruncate64 = SYS_FTRUNCATE64.ok_or(Errno::ENOSYS)?;
 
@@ -555,3 +567,30 @@ pub(crate) fn safe_unlinkat<Fd: AsFd>(
 
     Errno::result(res).map(drop)
 }
+
+/// Safe linkat(2) confined by syscall cookies.
+#[inline(always)]
+pub(crate) fn safe_linkat<Fd1: AsFd, Fd2: AsFd>(
+    olddirfd: Fd1,
+    oldpath: &XPath,
+    newdirfd: Fd2,
+    newpath: &XPath,
+    flag: AtFlags,
+) -> Result<(), Errno> {
+    let res = oldpath.with_nix_path(|oldcstr| {
+        // SAFETY: In libc we trust.
+        newpath.with_nix_path(|newcstr| unsafe {
+            libc::syscall(
+                libc::SYS_linkat,
+                olddirfd.as_fd().as_raw_fd(),
+                oldcstr.as_ptr(),
+                newdirfd.as_fd().as_raw_fd(),
+                newcstr.as_ptr(),
+                flag.bits(),
+                *LINKAT_COOKIE_ARG5,
+            )
+        })
+    })??;
+
+    Errno::result(res).map(drop)
+}
