@@ -9,7 +9,7 @@
 use std::{
     hash::BuildHasher,
     io::{IoSlice, Read},
-    os::fd::{AsFd, AsRawFd, BorrowedFd, FromRawFd, OwnedFd, RawFd},
+    os::fd::{AsFd, AsRawFd, BorrowedFd, FromRawFd, IntoRawFd, OwnedFd, RawFd},
 };
 
 #[allow(clippy::disallowed_types)]
@@ -21,26 +21,27 @@ use lexis::ToName;
 use memchr::arch::all::is_equal;
 use nix::{
     errno::Errno,
-    fcntl::{splice, tee, OFlag, SpliceFFlags},
-    sys::socket::{
-        accept4, bind, send, sendmsg, setsockopt, socket, sockopt::AlgSetKey, AddressFamily,
-        AlgAddr, ControlMessage, MsgFlags, SockFlag, SockType,
+    fcntl::{open, splice, tee, OFlag, SpliceFFlags},
+    sys::{
+        socket::{
+            accept4, bind, send, sendmsg, setsockopt, socket, sockopt::AlgSetKey, AddressFamily,
+            AlgAddr, ControlMessage, MsgFlags, SockFlag, SockType,
+        },
+        stat::Mode,
     },
     unistd::{lseek64, pipe2, read, write, Whence},
 };
-use procfs::{crypto as proc_crypto, SelfTest, Type};
-use secure_string::SecureBytes;
+use procfs_core::{SelfTest, Type};
 use sha1::Sha1;
 use sha3::{Digest, Sha3_256, Sha3_384, Sha3_512};
+use zeroize::Zeroizing;
 
 use crate::{
     config::*,
     err::SydResult,
-    fs::{
-        create_memfd, fillrandom, mkstempat, retry_on_eintr, set_append, set_nonblock,
-        MFD_ALLOW_SEALING, MFD_NOEXEC_SEAL,
-    },
+    fs::{create_memfd, fillrandom, mkstempat, retry_on_eintr, set_append, set_nonblock},
     path::XPath,
+    proc::proc_crypto_read,
 };
 
 /// Defines hash functions supported by Syd.
@@ -67,13 +68,13 @@ impl TryFrom<usize> for HashAlgorithm {
 
     fn try_from(len: usize) -> Result<Self, Self::Error> {
         match len {
-            4 => Ok(HashAlgorithm::Crc32),
-            8 => Ok(HashAlgorithm::Crc64),
-            16 => Ok(HashAlgorithm::Md5),
-            20 => Ok(HashAlgorithm::Sha1),
-            32 => Ok(HashAlgorithm::Sha256),
-            48 => Ok(HashAlgorithm::Sha384),
-            64 => Ok(HashAlgorithm::Sha512),
+            4 => Ok(Self::Crc32),
+            8 => Ok(Self::Crc64),
+            16 => Ok(Self::Md5),
+            20 => Ok(Self::Sha1),
+            32 => Ok(Self::Sha256),
+            48 => Ok(Self::Sha384),
+            64 => Ok(Self::Sha512),
             _ => Err(Errno::EINVAL),
         }
     }
@@ -105,21 +106,20 @@ pub const SENDFILE_MAX: usize = 0x7ffff000;
 
 /// Key holds the AES encryption key.
 ///
-/// This struct ensures that the key doesn't get swapped out and is
-/// securely zeroized when it is dropped.
-#[derive(Debug)]
-pub struct Key(SecureBytes);
+/// This struct ensures that the key is securely zeroized,
+/// when it is dropped.
+pub struct Key(Zeroizing<[u8; KEY_SIZE]>);
 
 impl Key {
     /// Creates a new Key with the given key data.
     pub fn new(key: [u8; KEY_SIZE]) -> Self {
-        Self(SecureBytes::from(key))
+        Self(Zeroizing::new(key))
     }
 
     /// Creates a random Key using the OS random number generator.
     pub fn random() -> Result<Self, Errno> {
-        let mut bytes = SecureBytes::new(vec![0; KEY_SIZE]);
-        fillrandom(bytes.unsecure_mut())?;
+        let mut bytes = Zeroizing::new([0u8; KEY_SIZE]);
+        fillrandom(bytes.as_mut())?;
         Ok(Self(bytes))
     }
 
@@ -132,7 +132,7 @@ impl Key {
 
     /// Returns a hex-encoded string of the KEY.
     pub fn as_hex(&self) -> String {
-        HEXLOWER.encode(self.0.unsecure())
+        HEXLOWER.encode(self.as_ref())
     }
 
     /// Check if the KEY is all zeros.
@@ -153,27 +153,26 @@ impl Key {
 
 impl AsRef<[u8]> for Key {
     fn as_ref(&self) -> &[u8] {
-        self.0.unsecure()
+        self.0.as_ref()
     }
 }
 
 impl AsMut<[u8]> for Key {
     fn as_mut(&mut self) -> &mut [u8] {
-        self.0.unsecure_mut()
+        self.0.as_mut()
     }
 }
 
 /// Key holds the AES IV
 ///
-/// This struct ensures that the IV doesn't get swapped out and is
-/// securely zeroized when it is dropped.
-#[derive(Debug)]
-pub struct IV(SecureBytes);
+/// This struct ensures that the IV is securely zeroized,
+/// when it is dropped.
+pub struct IV(Zeroizing<[u8; IV_SIZE]>);
 
 impl IV {
     /// Creates a new IV with the given key data.
     pub fn new(iv: [u8; IV_SIZE]) -> Self {
-        Self(SecureBytes::from(iv))
+        Self(Zeroizing::new(iv))
     }
 
     /// Creates a random IV using the OS random number generator.
@@ -182,8 +181,12 @@ impl IV {
     /// random bytes from AT_RANDOM is used instead.
     pub fn random() -> Self {
         let atrnd = get_at_random();
-        let mut bytes = SecureBytes::new(atrnd[..IV_SIZE].to_vec());
-        let _ = fillrandom(bytes.unsecure_mut());
+        let mut bytes = [0u8; IV_SIZE];
+        bytes.copy_from_slice(&atrnd[..IV_SIZE]);
+
+        let mut bytes = Zeroizing::new(bytes);
+        let _ = fillrandom(bytes.as_mut());
+
         Self(bytes)
     }
 
@@ -196,7 +199,7 @@ impl IV {
 
     /// Returns a hex-encoded string of the IV.
     pub fn as_hex(&self) -> String {
-        HEXLOWER.encode(self.0.unsecure())
+        HEXLOWER.encode(self.as_ref())
     }
 
     /// Check if the IV is all zeros.
@@ -230,7 +233,7 @@ impl IV {
         let mut ctr = ctr / BLOCK_SIZE as u64;
 
         // Access the IV bytes for modification.
-        let val = self.0.unsecure_mut();
+        let val = self.as_mut();
 
         // Process each byte of the IV from least significant to most
         // significant. This is because we are effectively treating the
@@ -260,19 +263,19 @@ impl IV {
 
 impl Clone for IV {
     fn clone(&self) -> Self {
-        IV(SecureBytes::from(self.0.unsecure()))
+        IV(self.0.clone())
     }
 }
 
 impl AsRef<[u8]> for IV {
     fn as_ref(&self) -> &[u8] {
-        self.0.unsecure()
+        self.0.as_ref()
     }
 }
 
 impl AsMut<[u8]> for IV {
     fn as_mut(&mut self) -> &mut [u8] {
-        self.0.unsecure_mut()
+        self.0.as_mut()
     }
 }
 
@@ -286,7 +289,7 @@ impl AsMut<[u8]> for IV {
 /// `Key` turns into `Alg` and is wiped from memory at startup.
 pub enum Secret {
     /// Encryption & Authentication sockets
-    Alg(OwnedFd, OwnedFd),
+    Alg(RawFd, RawFd),
     /// Uninitialized master key
     Key(Key),
 }
@@ -312,16 +315,16 @@ impl Secret {
         };
         let enc_fd = {
             let enc_key = key.derive(salt, info_enc);
-            aes_ctr_setup(&enc_key)
+            aes_ctr_setup(enc_key.as_ref())
         }?;
         let tag_fd = {
             let tag_key = key.derive(salt, info_tag);
-            hmac_sha256_setup(&tag_key.0)
+            hmac_sha256_setup(tag_key.as_ref())
         }?;
 
         // SAFETY: Free key from secure memory,
         // and replace it with the KCAPI connection.
-        *self = Self::Alg(enc_fd, tag_fd);
+        *self = Self::Alg(enc_fd.into_raw_fd(), tag_fd.into_raw_fd());
 
         Ok(())
     }
@@ -390,7 +393,13 @@ pub fn hash<R: Read>(mut reader: R, func: HashAlgorithm) -> SydResult<Vec<u8>> {
 
 /// Returns a concise summary of hmac(sha256) shash support in the kernel.
 pub fn hmac_sha256_info() -> String {
-    match proc_crypto() {
+    #[allow(clippy::disallowed_methods)]
+    let fd = match open("/proc/crypto", OFlag::O_RDONLY, Mode::empty()) {
+        Ok(fd) => fd,
+        Err(e) => return format!("HMAC-SHA256: failed to open /proc/crypto: {e}!"),
+    };
+
+    match proc_crypto_read(fd) {
         Err(e) => format!("HMAC-SHA256: failed to read /proc/crypto: {e}!"),
         Ok(table) => {
             if let Some(blocks) = table.crypto_blocks.get("hmac(sha256)") {
@@ -443,11 +452,11 @@ blocksize {}B; digestsize {}B.",
 /// # Returns
 ///
 /// * `Result<OwnedFd, Errno>` - The file descriptor for the socket on success, or an error.
-pub fn hmac_sha256_setup(key: &SecureBytes) -> Result<OwnedFd, Errno> {
+pub fn hmac_sha256_setup(key: &[u8]) -> Result<OwnedFd, Errno> {
     // SAFETY: The key must be hashed with sha256 if its larger than the
     // block size, see Test Case 6 of RFC4231. We simply validate here,
     // and expect the caller to handle it.
-    if key.unsecure().len() > SHA256_BLOCK_SIZE {
+    if key.len() > SHA256_BLOCK_SIZE {
         return Err(Errno::EINVAL);
     }
 
@@ -466,7 +475,7 @@ pub fn hmac_sha256_setup(key: &SecureBytes) -> Result<OwnedFd, Errno> {
     bind(sock.as_raw_fd(), &addr)?;
 
     // Set the encryption key.
-    setsockopt(&sock, AlgSetKey::default(), &key.unsecure())?;
+    setsockopt(&sock, AlgSetKey::default(), &key)?;
 
     Ok(sock)
 }
@@ -506,9 +515,14 @@ pub fn hmac_sha256_feed<S: AsRawFd>(sock: &S, chunk: &[u8], more: bool) -> Resul
 }
 
 /// Finishes the HMAC-SHA256 authentication and reads authentication tag.
-pub fn hmac_sha256_fini<Fd: AsFd>(sock: Fd) -> Result<SecureBytes, Errno> {
-    let mut data = SecureBytes::new(vec![0u8; SHA256_DIGEST_SIZE]);
-    let buf = data.unsecure_mut();
+pub fn hmac_sha256_fini<Fd: AsFd>(sock: Fd) -> Result<Zeroizing<Vec<u8>>, Errno> {
+    let mut data = Vec::new();
+    data.try_reserve(SHA256_DIGEST_SIZE)
+        .or(Err(Errno::ENOMEM))?;
+    data.resize(SHA256_DIGEST_SIZE, 0);
+
+    let mut data = Zeroizing::new(data);
+    let buf: &mut [u8] = data.as_mut();
 
     let mut nread = 0;
     while nread < SHA256_DIGEST_SIZE {
@@ -526,7 +540,13 @@ pub fn hmac_sha256_fini<Fd: AsFd>(sock: Fd) -> Result<SecureBytes, Errno> {
 
 /// Returns a concise summary of ctr(aes) skcipher support in the kernel.
 pub fn aes_ctr_info() -> String {
-    match proc_crypto() {
+    #[allow(clippy::disallowed_methods)]
+    let fd = match open("/proc/crypto", OFlag::O_RDONLY, Mode::empty()) {
+        Ok(fd) => fd,
+        Err(e) => return format!("AES-CTR: failed to open /proc/crypto: {e}!"),
+    };
+
+    match proc_crypto_read(fd) {
         Err(e) => format!("AES-CTR: failed to read /proc/crypto: {e}!"),
         Ok(table) => {
             if let Some(blocks) = table.crypto_blocks.get("ctr(aes)") {
@@ -576,7 +596,7 @@ key {}–{}B; iv {}B; chunk {}B; walk {}B.",
 }
 
 /// Sets up the AES-CTR encryption/decryption using the Kernel crypto API.
-pub fn aes_ctr_setup(key: &Key) -> Result<OwnedFd, Errno> {
+pub fn aes_ctr_setup(key: &[u8]) -> Result<OwnedFd, Errno> {
     // Create the socket for the AF_ALG interface
     let sock = socket(
         AddressFamily::Alg,
@@ -592,7 +612,7 @@ pub fn aes_ctr_setup(key: &Key) -> Result<OwnedFd, Errno> {
     bind(sock.as_raw_fd(), &addr)?;
 
     // Set the encryption key.
-    setsockopt(&sock, AlgSetKey::default(), &key.as_ref())?;
+    setsockopt(&sock, AlgSetKey::default(), &key)?;
 
     Ok(sock)
 }
@@ -630,16 +650,20 @@ pub fn aes_ctr_enc<S: AsRawFd>(
     let iov = [IoSlice::new(chunk)];
 
     // Prepare the control message for the IV.
-    let mut cmsgs = vec![ControlMessage::AlgSetOp(&nix::libc::ALG_OP_ENCRYPT)];
-    if let Some(iv) = iv {
-        cmsgs.push(ControlMessage::AlgSetIv(iv.as_ref()));
-    }
+    let cmsgs = if let Some(iv) = iv {
+        &[
+            ControlMessage::AlgSetOp(&libc::ALG_OP_ENCRYPT),
+            ControlMessage::AlgSetIv(iv.as_ref()),
+        ][..]
+    } else {
+        &[ControlMessage::AlgSetOp(&libc::ALG_OP_ENCRYPT)][..]
+    };
 
     // Determine the flags for the sendmsg operation.
     let flags = if more { MSG_MORE } else { MsgFlags::empty() };
 
     // Send the message with the IV and data
-    retry_on_eintr(|| sendmsg::<AlgAddr>(sock.as_raw_fd(), &iov, cmsgs.as_slice(), flags, None))
+    retry_on_eintr(|| sendmsg::<AlgAddr>(sock.as_raw_fd(), &iov, cmsgs, flags, None))
 }
 
 /// Decrypts a chunk of data using the initialized AES-CTR socket.
@@ -653,22 +677,30 @@ pub fn aes_ctr_dec<S: AsRawFd>(
     let iov = [IoSlice::new(chunk)];
 
     // Prepare the control message for IV.
-    let mut cmsgs = vec![ControlMessage::AlgSetOp(&nix::libc::ALG_OP_DECRYPT)];
-    if let Some(iv) = iv {
-        cmsgs.push(ControlMessage::AlgSetIv(iv.as_ref()));
-    }
+    let cmsgs = if let Some(iv) = iv {
+        &[
+            ControlMessage::AlgSetOp(&libc::ALG_OP_DECRYPT),
+            ControlMessage::AlgSetIv(iv.as_ref()),
+        ][..]
+    } else {
+        &[ControlMessage::AlgSetOp(&libc::ALG_OP_DECRYPT)][..]
+    };
 
     // Determine the flags for the sendmsg operation
     let flags = if more { MSG_MORE } else { MsgFlags::empty() };
 
     // Send the message with the IV and data
-    retry_on_eintr(|| sendmsg::<AlgAddr>(sock.as_raw_fd(), &iov, cmsgs.as_slice(), flags, None))
+    retry_on_eintr(|| sendmsg::<AlgAddr>(sock.as_raw_fd(), &iov, cmsgs, flags, None))
 }
 
 /// Finishes the AES-CTR {en,de}cryption and reads the {de,en}crypted data.
-pub fn aes_ctr_fini<Fd: AsFd>(sock: Fd, size: usize) -> Result<SecureBytes, Errno> {
-    let mut data = SecureBytes::new(vec![0u8; size]);
-    let buf = data.unsecure_mut();
+pub fn aes_ctr_fini<Fd: AsFd>(sock: Fd, size: usize) -> Result<Zeroizing<Vec<u8>>, Errno> {
+    let mut data = Vec::new();
+    data.try_reserve(size).or(Err(Errno::ENOMEM))?;
+    data.resize(size, 0);
+
+    let mut data = Zeroizing::new(data);
+    let buf: &mut [u8] = data.as_mut();
 
     let mut nread = 0;
     while nread < size {
@@ -692,7 +724,6 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
     fd: Fd,
     flags: OFlag,
     tmp: Option<RawFd>,
-    restrict_memfd: bool,
 ) -> Result<Option<(OwnedFd, IV)>, Errno> {
     let (aes_fd, mac_fd) = setup_fds;
 
@@ -729,10 +760,9 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
             return Ok(None);
         }
 
-        // Read HMAC tag.
-        // SAFETY: We don't swap HMAC out!
-        let mut hmac_tag = SecureBytes::from([0u8; HMAC_TAG_SIZE]);
-        let buf = hmac_tag.unsecure_mut();
+        // Read HMAC tag, zeroize on drop.
+        let mut hmac_tag = Zeroizing::new([0u8; HMAC_TAG_SIZE]);
+        let buf = hmac_tag.as_mut();
         let mut nread = 0;
         while nread < buf.len() {
             #[allow(clippy::arithmetic_side_effects)]
@@ -747,8 +777,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
             }
         }
 
-        // Read IV.
-        // SAFETY: We don't swap IV out!
+        // Read IV, zeroized on drop.
         let mut iv = IV::new([0u8; IV_SIZE]);
         let buf = iv.as_mut();
         let mut nread = 0;
@@ -771,17 +800,9 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
     let dst_fd = if let Some(tmp) = tmp {
         // SAFETY: `tmp' is alive for the duration of the Syd sandbox.
         let tmp = unsafe { BorrowedFd::borrow_raw(tmp) };
-        mkstempat(tmp, XPath::from_bytes(b""))
+        mkstempat(tmp, XPath::from_bytes(b"syd-aes-"))
     } else {
-        // Note, MFD_ALLOW_SEALING is implied for MFD_NOEXEC_SEAL.
-        create_memfd(
-            b"syd-aes\0",
-            if restrict_memfd {
-                MFD_NOEXEC_SEAL
-            } else {
-                MFD_ALLOW_SEALING
-            },
-        )
+        create_memfd(b"syd-aes\0", *SAFE_MFD_FLAGS)
     }?;
 
     let iv = if let Some((iv, hmac_tag)) = iv_and_tag {
@@ -961,7 +982,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
 
         // Compare computed HMAC with the HMAC tag read from the file.
         // SAFETY: Compare in constant time!
-        if !constant_time_eq(computed_hmac.unsecure(), hmac_tag.unsecure()) {
+        if !constant_time_eq(computed_hmac.as_slice(), hmac_tag.as_slice()) {
             // HMAC verification failed.
             return Err(Errno::EBADMSG);
         }
@@ -1022,7 +1043,7 @@ pub fn aes_ctr_flush<S: AsFd, F: AsFd>(
     buf: &mut [u8],
     size: usize,
 ) -> Result<usize, Errno> {
-    debug_assert!(buf.len() >= size);
+    assert!(buf.len() >= size);
 
     // Read from the socket.
     let mut nread = 0;
@@ -1055,7 +1076,7 @@ pub fn aes_ctr_flush<S: AsFd, F: AsFd>(
 pub fn get_at_random() -> &'static [u8; 16] {
     // SAFETY: In libc we trust.
     unsafe {
-        let ptr = nix::libc::getauxval(nix::libc::AT_RANDOM) as *const u8;
+        let ptr = libc::getauxval(libc::AT_RANDOM) as *const u8;
         assert!(!ptr.is_null(), "AT_RANDOM not found");
         &*(ptr as *const [u8; 16])
     }
@@ -1142,6 +1163,14 @@ impl BuildHasher for SydRandomState {
     }
 }
 
+/// Convenience alias for HashMap with `SydRandomState`
+#[allow(clippy::disallowed_types)]
+pub type SydHashMap<K, V> = std::collections::HashMap<K, V, SydRandomState>;
+
+/// Convenience alias for HashSet with `SydRandomState`
+#[allow(clippy::disallowed_types)]
+pub type SydHashSet<K> = std::collections::HashSet<K, SydRandomState>;
+
 #[cfg(test)]
 mod tests {
     use std::io::Cursor;
@@ -1444,14 +1473,11 @@ mod tests {
         let mut errors = Vec::new();
 
         for (i, test_case) in HMAC_TEST_CASES.iter().enumerate() {
-            let key_bytes = test_case.0;
+            let key = test_case.0;
             let data = test_case.1;
             let expected_hmac = test_case.2.to_lowercase();
 
-            // Step 1: Prepare the key as SecureBytes
-            let key = SecureBytes::new(key_bytes.to_vec());
-
-            if key_bytes.len() > SHA256_BLOCK_SIZE {
+            if key.len() > SHA256_BLOCK_SIZE {
                 // Expect hmac_sha256_setup to return EINVAL for keys longer than block size
                 match hmac_sha256_setup(&key) {
                     Err(Errno::EINVAL) => {
@@ -1461,7 +1487,7 @@ mod tests {
                         errors.push(format!(
                             "Test case {}: Expected EINVAL for key length {}, but setup succeeded.",
                             i + 1,
-                            key_bytes.len()
+                            key.len()
                         ));
                     }
                     Err(e) => {
@@ -1526,7 +1552,7 @@ mod tests {
             };
 
             // Step 6: Convert the HMAC tag to a hex string
-            let computed_hex = HEXLOWER.encode(hmac_result.unsecure());
+            let computed_hex = HEXLOWER.encode(hmac_result.as_slice());
 
             // Step 7: Compare with the expected output
             if expected_hmac.len() < 64 {
@@ -1804,7 +1830,7 @@ mod tests {
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
 
-        match aes_ctr_setup(&key).map(drop) {
+        match aes_ctr_setup(key.as_ref()).map(drop) {
             Ok(()) => {}
             Err(Errno::EAFNOSUPPORT) => {
                 // KCAPI not supported, skip.
@@ -1819,7 +1845,7 @@ mod tests {
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
 
-        let setup_fd = match aes_ctr_setup(&key) {
+        let setup_fd = match aes_ctr_setup(key.as_ref()) {
             Ok(fd) => fd,
             Err(Errno::EAFNOSUPPORT) => {
                 // KCAPI not supported, skip.
@@ -1840,7 +1866,7 @@ mod tests {
         let iv = IV::random();
         assert!(!iv.is_zero(), "iv is all zeros!");
 
-        let setup_fd = match aes_ctr_setup(&key) {
+        let setup_fd = match aes_ctr_setup(key.as_ref()) {
             Ok(fd) => fd,
             Err(Errno::EAFNOSUPPORT) => {
                 // KCAPI not supported, skip.
@@ -1858,17 +1884,16 @@ mod tests {
         assert_eq!(encrypted_size, data.len());
 
         let encrypted_data = aes_ctr_fini(&sock_enc, encrypted_size).unwrap();
-        assert_eq!(encrypted_data.unsecure().len(), encrypted_size,);
+        assert_eq!(encrypted_data.len(), encrypted_size,);
         drop(sock_enc);
 
         let sock_dec = aes_ctr_init(&setup_fd, false).unwrap();
         aes_ctr_dec(&sock_dec, &[], Some(&iv), true).unwrap();
-        let decrypted_size =
-            aes_ctr_dec(&sock_dec, &encrypted_data.unsecure(), None, false).unwrap();
+        let decrypted_size = aes_ctr_dec(&sock_dec, &encrypted_data.as_ref(), None, false).unwrap();
         assert_eq!(decrypted_size, encrypted_size);
 
         let decrypted_data = aes_ctr_fini(&sock_dec, encrypted_size).unwrap();
-        assert_eq!(decrypted_data.unsecure(), data);
+        assert_eq!(decrypted_data.as_slice(), data);
     }
 
     #[test]
@@ -1879,7 +1904,7 @@ mod tests {
         let iv = IV::random();
         assert!(!iv.is_zero(), "iv is all zeros!");
 
-        let setup_fd = match aes_ctr_setup(&key) {
+        let setup_fd = match aes_ctr_setup(key.as_ref()) {
             Ok(fd) => fd,
             Err(Errno::EAFNOSUPPORT) => {
                 // KCAPI not supported, skip.
@@ -1922,19 +1947,19 @@ mod tests {
         eprintln!("SETTING IV");
         aes_ctr_dec(&sock_dec, &[], Some(&iv), true).unwrap();
         eprintln!("WRITING ENCRYPTED DATA");
-        let dec_result = aes_ctr_dec(&sock_dec, &encrypted_data.unsecure(), None, false).unwrap();
+        let dec_result = aes_ctr_dec(&sock_dec, &encrypted_data.as_ref(), None, false).unwrap();
         assert_eq!(dec_result, total_encrypted_size);
 
         eprintln!("FINALIZING DECRYPTION");
         let decrypted_data = aes_ctr_fini(&sock_dec, total_encrypted_size).unwrap();
         assert_eq!(
-            decrypted_data.unsecure().len(),
+            decrypted_data.len(),
             total_encrypted_size,
             "{:?}",
-            decrypted_data.unsecure()
+            decrypted_data.as_slice()
         );
         let original_data: Vec<u8> = data_chunks.concat();
-        assert_eq!(decrypted_data.unsecure(), original_data.as_slice());
+        assert_eq!(decrypted_data.as_slice(), original_data.as_slice());
     }
 
     #[test]
@@ -1980,12 +2005,12 @@ mod tests {
         let encrypted_memfd = create_memfd(b"syd\0", 0).unwrap();
         let nwrite = write(encrypted_memfd.as_fd(), CRYPT_MAGIC).unwrap();
         assert_eq!(nwrite, CRYPT_MAGIC.len());
-        let nwrite = write(encrypted_memfd.as_fd(), hmac_tag.unsecure()).unwrap();
+        let nwrite = write(encrypted_memfd.as_fd(), hmac_tag.as_ref()).unwrap();
         assert_eq!(nwrite, HMAC_TAG_SIZE);
         let nwrite = write(encrypted_memfd.as_fd(), iv.as_ref()).unwrap();
         assert_eq!(nwrite, IV_SIZE);
-        let nwrite = write(encrypted_memfd.as_fd(), &encrypted_data.unsecure()).unwrap();
-        assert_eq!(nwrite, encrypted_data.unsecure().len());
+        let nwrite = write(encrypted_memfd.as_fd(), &encrypted_data.as_ref()).unwrap();
+        assert_eq!(nwrite, encrypted_data.len());
 
         // Decrypt the data directly into a memfd with zero-copy.
         let sock_dec = aes_ctr_init(&setup_enc, false).unwrap();
@@ -1995,7 +2020,6 @@ mod tests {
             &encrypted_memfd,
             OFlag::empty(),
             Some(tmp_dir.as_raw_fd()),
-            false,
         ) {
             Ok(fd) => fd.unwrap(),
             Err(Errno::EOPNOTSUPP) => {
