@@ -7,21 +7,22 @@
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
+    fmt,
     fs::File,
     io::Seek,
     os::fd::{AsRawFd, RawFd},
 };
 
 use libseccomp::ScmpNotifResp;
-use memchr::arch::all::is_prefix;
 use nix::{errno::Errno, fcntl::OFlag};
+use serde::{Serialize, Serializer};
 
 use crate::{
     config::{PAGE_SIZE, PROC_FILE},
     elf::ExecutableFile,
     error,
-    fs::{is_writable_fd, safe_open_magicsym, CanonicalPath},
-    hook::{RemoteProcess, UNotifyEventRequest},
+    fs::{fd_status_flags, safe_open_magicsym, CanonicalPath},
+    hook::UNotifyEventRequest,
     kernel::sandbox_path,
     path::XPathBuf,
     proc::{proc_mem_limit, proc_statm},
@@ -31,25 +32,64 @@ use crate::{
 
 const PROT_EXEC: u64 = libc::PROT_EXEC as u64;
 const MAP_ANONYMOUS: u64 = libc::MAP_ANONYMOUS as u64;
+const MAP_SHARED: u64 = libc::MAP_SHARED as u64;
+
+// `MemSyscall` represents possible memory family system calls.
+//
+// This list of memory family system calls are: brk(2), mmap(2),
+// mmap2(2), and mremap(2).
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+enum MemSyscall {
+    Brk,
+    Mmap,
+    Mmap2,
+    Mremap,
+}
+
+impl MemSyscall {
+    const fn is_mmap(self) -> bool {
+        matches!(self, Self::Mmap | Self::Mmap2)
+    }
+
+    const fn caps(self) -> Capability {
+        match self {
+            Self::Brk | Self::Mremap => Capability::CAP_MEM,
+            Self::Mmap | Self::Mmap2 => Capability::CAP_MMAP,
+        }
+    }
+}
+
+impl fmt::Display for MemSyscall {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let name = match self {
+            Self::Brk => "brk",
+            Self::Mmap => "mmap",
+            Self::Mmap2 => "mmap2",
+            Self::Mremap => "mremap",
+        };
+        f.write_str(name)
+    }
+}
+
+impl Serialize for MemSyscall {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_str(&self.to_string())
+    }
+}
 
 pub(crate) fn sys_brk(request: UNotifyEventRequest) -> ScmpNotifResp {
-    syscall_mem_handler(request, "brk", Capability::CAP_MEM)
+    syscall_mem_handler(request, MemSyscall::Brk)
 }
 
 pub(crate) fn sys_mmap(request: UNotifyEventRequest) -> ScmpNotifResp {
-    syscall_mem_handler(
-        request,
-        "mmap",
-        Capability::CAP_MEM | Capability::CAP_EXEC | Capability::CAP_FORCE | Capability::CAP_TPE,
-    )
+    syscall_mem_handler(request, MemSyscall::Mmap)
 }
 
 pub(crate) fn sys_mmap2(request: UNotifyEventRequest) -> ScmpNotifResp {
-    syscall_mem_handler(
-        request,
-        "mmap2",
-        Capability::CAP_MEM | Capability::CAP_EXEC | Capability::CAP_FORCE | Capability::CAP_TPE,
-    )
+    syscall_mem_handler(request, MemSyscall::Mmap2)
 }
 
 pub(crate) fn sys_mremap(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -61,16 +101,13 @@ pub(crate) fn sys_mremap(request: UNotifyEventRequest) -> ScmpNotifResp {
         // No pointer dereference in size check.
         return unsafe { request.continue_syscall() };
     }
-    syscall_mem_handler(request, "mremap", Capability::CAP_MEM)
+    syscall_mem_handler(request, MemSyscall::Mremap)
 }
 
 #[allow(clippy::cognitive_complexity)]
-fn syscall_mem_handler(
-    request: UNotifyEventRequest,
-    name: &str,
-    caps: Capability,
-) -> ScmpNotifResp {
+fn syscall_mem_handler(request: UNotifyEventRequest, syscall: MemSyscall) -> ScmpNotifResp {
     let req = request.scmpreq;
+    let caps = syscall.caps();
 
     // Get mem & vm max.
     let sandbox = request.get_sandbox();
@@ -83,14 +120,16 @@ fn syscall_mem_handler(
     let mem_max = sandbox.mem_max;
     let mem_vm_max = sandbox.mem_vm_max;
     let mem_act = sandbox.default_action(Capability::CAP_MEM);
-    let restrict_memory = !sandbox.allow_unsafe_memory();
-    let restrict_stack = !sandbox.allow_unsafe_stack();
+    let restrict_memory = !sandbox.flags.allow_unsafe_memory();
+    let restrict_stack = !sandbox.flags.allow_unsafe_stack();
+    let restrict_append_only = sandbox.has_append() || sandbox.enabled(Capability::CAP_CRYPT);
 
     if !exec
         && !force
         && !tpe
         && !restrict_memory
         && !restrict_stack
+        && !restrict_append_only
         && (!mem || (mem_max == 0 && mem_vm_max == 0))
     {
         // SAFETY: No pointer dereference in security check.
@@ -98,53 +137,83 @@ fn syscall_mem_handler(
         return unsafe { request.continue_syscall() };
     }
 
-    if (exec || force || tpe || restrict_memory || restrict_stack)
-        && is_prefix(name.as_bytes(), b"mmap")
+    let name = syscall.to_string();
+
+    let check_exec = syscall.is_mmap()
+        && (exec || force || tpe || restrict_memory || restrict_stack)
         && req.data.args[2] & PROT_EXEC != 0
-        && req.data.args[3] & MAP_ANONYMOUS == 0
-    {
-        // Check file descriptor for Exec access.
-        // Read and Write were already checked at open(2).
+        && req.data.args[3] & MAP_ANONYMOUS == 0;
+    let check_append_only = restrict_append_only && req.data.args[3] & MAP_SHARED != 0;
+
+    // Get the file descriptor before access check.
+    let fd = if check_exec || check_append_only {
         #[allow(clippy::cast_possible_truncation)]
         let remote_fd = req.data.args[4] as RawFd;
         if remote_fd < 0 {
             return request.fail_syscall(Errno::EBADF);
         }
 
-        // SAFETY: Get the file descriptor before access check
-        // as it may change after which is a TOCTOU vector.
-        let fd = match request.get_fd(remote_fd) {
-            Ok(fd) => fd,
+        match request.get_fd(remote_fd) {
+            Ok(fd) => Some(fd),
             Err(_) => return request.fail_syscall(Errno::EBADF),
-        };
+        }
+    } else {
+        None
+    };
 
+    #[allow(clippy::disallowed_methods)]
+    let oflags = if check_append_only || (check_exec && restrict_memory) {
+        fd_status_flags(fd.as_ref().unwrap()).ok()
+    } else {
+        None
+    };
+
+    if check_append_only {
+        // Prevent shared mappings on writable append-only fds.
+        let deny = oflags
+            .map(|fl| {
+                fl.contains(OFlag::O_APPEND)
+                    && (fl.contains(OFlag::O_RDWR) || fl.contains(OFlag::O_WRONLY))
+            })
+            .unwrap_or(true);
+
+        if deny {
+            return request.fail_syscall(Errno::EACCES);
+        }
+    }
+
+    if check_exec {
         // Step 1: Check if file is open for write,
         // but set as PROT_READ|PROT_EXEC which breaks W^X!
         // We do not need to check for PROT_WRITE here as
         // this is already enforced at kernel-level when
         // trace/allow_unsafe_memory:1 is not set at startup.
-        if restrict_memory && is_writable_fd(&fd).unwrap_or(true) {
-            return request.fail_syscall(Errno::EACCES);
+        if restrict_memory {
+            let deny = oflags
+                .map(|fl| fl.contains(OFlag::O_RDWR) || fl.contains(OFlag::O_WRONLY))
+                .unwrap_or(true);
+
+            if deny {
+                return request.fail_syscall(Errno::EACCES);
+            }
         }
 
-        let mut path = match CanonicalPath::new_fd(fd.into(), req.pid(), remote_fd) {
+        #[allow(clippy::disallowed_methods)]
+        let mut path = match CanonicalPath::new_fd(fd.unwrap().into(), req.pid()) {
             Ok(path) => path,
             Err(errno) => return request.fail_syscall(errno),
         };
 
         // Step 2: Check for Exec sandboxing.
         if exec {
-            // Unused when request.is_some()
-            let process = RemoteProcess::new(request.scmpreq.pid());
-
             if let Err(errno) = sandbox_path(
                 Some(&request),
                 &sandbox,
-                &process,
+                request.scmpreq.pid(), // Unused when request.is_some()
                 path.abs(),
                 Capability::CAP_EXEC,
                 false,
-                name,
+                &name,
             ) {
                 return request.fail_syscall(errno);
             }
@@ -152,18 +221,22 @@ fn syscall_mem_handler(
 
         // Step 3: Check for TPE sandboxing.
         if tpe {
-            let action = sandbox.check_tpe(path.abs());
+            // MUST_PATH ensures path.dir is Some.
+            #[allow(clippy::disallowed_methods)]
+            let file = path.dir.as_ref().unwrap();
+            let (action, msg) = sandbox.check_tpe(file, path.abs());
             if !matches!(action, Action::Allow | Action::Filter) {
+                let msg = msg.as_deref().unwrap_or("?");
                 if verbose {
                     error!("ctx": "trusted_path_execution",
-                        "err": "library load from untrusted path blocked",
-                        "sys": name, "path": &path,
+                        "err": format!("library load from untrusted path blocked: {msg}"),
+                        "sys": &name, "path": &path,
                         "req": &request,
                         "tip": "move the library to a safe location or use `sandbox/tpe:off'");
                 } else {
                     error!("ctx": "trusted_path_execution",
-                        "err": "library load from untrusted path blocked",
-                        "sys": name, "path": &path,
+                        "err": format!("library load from untrusted path blocked: {msg}"),
+                        "sys": &name, "path": &path,
                         "pid": request.scmpreq.pid,
                         "tip": "move the library to a safe location or use `sandbox/tpe:off'");
                 }
@@ -184,13 +257,11 @@ fn syscall_mem_handler(
         if force || restrict_stack {
             // The following checks require the contents of the file.
             // SAFETY:
-            // 1. Reopen the file via `/proc/self/fd` to avoid sharing the file offset.
+            // 1. Reopen the file via `/proc/thread-self/fd` to avoid sharing the file offset.
             // 2. `path` is a remote-fd transfer which asserts `path.dir` is Some.
             #[allow(clippy::disallowed_methods)]
             let fd = path.dir.take().unwrap();
-
-            let mut pfd = XPathBuf::from("self/fd");
-            pfd.push_fd(fd.as_raw_fd());
+            let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
 
             let mut file =
                 match safe_open_magicsym(PROC_FILE(), &pfd, OFlag::O_RDONLY).map(File::from) {
@@ -214,14 +285,14 @@ fn syscall_mem_handler(
                             if verbose {
                                 error!("ctx": "check_lib",
                                     "err": "library load with executable stack blocked",
-                                    "sys": name, "path": path.abs(),
+                                    "sys": &name, "path": path.abs(),
                                     "tip": "configure `trace/allow_unsafe_stack:1'",
                                     "lib": format!("{exe}"),
                                     "req": &request);
                             } else {
                                 error!("ctx": "check_lib",
                                     "err": "library load with executable stack blocked",
-                                    "sys": name, "path": path.abs(),
+                                    "sys": &name, "path": path.abs(),
                                     "tip": "configure `trace/allow_unsafe_stack:1'",
                                     "lib": format!("{exe}"),
                                     "pid": request.scmpreq.pid);
@@ -251,12 +322,12 @@ fn syscall_mem_handler(
                         if !matches!(action, Action::Allow | Action::Filter) {
                             if verbose {
                                 warn!("ctx": "verify_lib", "act": action,
-                                    "sys": name, "path": path.abs(),
+                                    "sys": &name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
-                                    "sys": name, "req": &request);
+                                    "sys": &name, "req": &request);
                             } else {
                                 warn!("ctx": "verify_lib", "act": action,
-                                    "sys": name, "path": path.abs(),
+                                    "sys": &name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                     "pid": request.scmpreq.pid);
                             }
@@ -277,13 +348,13 @@ fn syscall_mem_handler(
                         if verbose {
                             error!("ctx": "verify_lib",
                                 "err": format!("system error during library checksum calculation: {errno}"),
-                                "sys": name, "path": path.abs(),
+                                "sys": &name, "path": path.abs(),
                                 "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                 "req": &request);
                         } else {
                             error!("ctx": "verify_lib",
                                 "err": format!("system error during library checksum calculation: {errno}"),
-                                "sys": name, "path": path.abs(),
+                                "sys": &name, "path": path.abs(),
                                 "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                 "pid": request.scmpreq.pid);
                         }
@@ -298,13 +369,13 @@ fn syscall_mem_handler(
                             if sandbox.verbose {
                                 error!("ctx": "verify_lib", "act": action,
                                     "err": format!("library checksum mismatch: {found} is not {expected}"),
-                                    "sys": name, "path": path.abs(),
+                                    "sys": &name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                     "req": &request);
                             } else {
                                 error!("ctx": "verify_lib", "act": action,
                                     "err": format!("library checksum mismatch: {found} is not {expected}"),
-                                    "sys": name, "path": path.abs(),
+                                    "sys": &name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                     "pid": request.scmpreq.pid);
                             }
@@ -354,12 +425,12 @@ fn syscall_mem_handler(
             if mem_act != Action::Filter {
                 if verbose {
                     warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
-                        "sys": name, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
+                        "sys": &name, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
                         "tip": "increase `mem/vm_max'",
                         "req": &request);
                 } else {
                     warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
-                        "sys": name, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
+                        "sys": &name, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
                         "tip": "increase `mem/vm_max'",
                         "pid": request.scmpreq.pid);
                 }
@@ -390,12 +461,12 @@ fn syscall_mem_handler(
                 if mem_act != Action::Filter {
                     if verbose {
                         warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
-                            "sys": name, "mem_max": mem_max,
+                            "sys": &name, "mem_max": mem_max,
                             "tip": "increase `mem/max'",
                             "req": &request);
                     } else {
                         warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
-                            "sys": name, "mem_max": mem_max,
+                            "sys": &name, "mem_max": mem_max,
                             "tip": "increase `mem/max'",
                             "pid": request.scmpreq.pid);
                     }
