@@ -15,8 +15,9 @@ use nix::{
 use crate::{
     caps,
     config::{GID_MIN, UID_MIN},
+    confine::safe_drop_cap,
     hook::UNotifyEventRequest,
-    safe_drop_cap, warn,
+    warn,
 };
 
 #[allow(clippy::cognitive_complexity)]
