@@ -7,7 +7,6 @@
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
-    collections::HashSet,
     fs::File,
     io::BufReader,
     os::{
@@ -22,12 +21,13 @@ use nix::{errno::Errno, NixPath};
 use crate::{
     compat::{fstatat64, statx, STATX_BASIC_STATS, STATX_MODE, STATX_TYPE},
     config::{MAGIC_PREFIX, MMAP_MIN_ADDR},
+    confine::scmp_arch_bits,
     fs::{is_sidechannel_device, parse_fd, CanonicalPath, FileInfo, FileType, FsFlags},
-    hook::{RemoteProcess, SysArg, SysFlags, UNotifyEventRequest},
+    hash::SydHashSet,
+    hook::{SysArg, SysFlags, UNotifyEventRequest},
     kernel::sandbox_path,
     path::XPath,
     sandbox::Capability,
-    scmp_arch_bits,
 };
 
 pub(crate) fn sys_stat(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -163,10 +163,6 @@ fn syscall_stat_handler(
     is32: bool,
 ) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
-        // Note: This is a virtual call handler,
-        // `sandbox` is an upgradable read lock with exclusive access.
-        // We'll either upgrade it or downgrade it based on magic lock.
-        // Exception: Sandbox lock had been set and there's no turning back.
         let req = request.scmpreq;
         let is_fd = arg.path.is_none();
         let sandbox = request.get_sandbox();
@@ -231,7 +227,7 @@ fn syscall_stat_handler(
                     Ok(remote_fd) => {
                         let fd = request.get_fd(remote_fd)?;
                         let file = BufReader::new(File::from(fd));
-                        let mut imap = HashSet::default();
+                        let mut imap = SydHashSet::default();
                         // SAFETY: parse_config() checks for the file name
                         // /dev/syd/load and disables config file include
                         // feature depending on this check.
@@ -276,7 +272,9 @@ fn syscall_stat_handler(
                 if let Ok(info) = FileInfo::from_fd(fd) {
                     #[allow(clippy::disallowed_methods)]
                     let files = request.crypt_map.as_ref().unwrap();
-                    for (enc_path, map) in &files.read().unwrap_or_else(|err| err.into_inner()).0 {
+                    for (enc_path, map) in
+                        files.read().unwrap_or_else(|err| err.into_inner()).iter()
+                    {
                         if info == map.info {
                             // Found underlying encrypted file for the memory fd.
                             // Note, we only ever attempt to encrypt regular files.
@@ -300,13 +298,10 @@ fn syscall_stat_handler(
                 && arg.path.is_some()
                 && !path.abs().starts_with(b"/memfd:syd-")
             {
-                // Unused when request.is_some()
-                let process = RemoteProcess::new(request.scmpreq.pid());
-
                 sandbox_path(
                     Some(&request),
                     &sandbox,
-                    &process,
+                    request.scmpreq.pid(), // Unused when request.is_some()
                     path.abs(),
                     Capability::CAP_STAT,
                     false,
@@ -321,20 +316,20 @@ fn syscall_stat_handler(
         //
         // Return ENOTDIR for non-directories with trailing slash.
         if let Some(file_type) = &path.typ {
-            if !matches!(file_type, FileType::Dir | FileType::MagicLnk(_, _))
+            if !matches!(file_type, FileType::Dir | FileType::MagicLnk(_))
                 && path.abs().last() == Some(b'/')
             {
                 return Err(Errno::ENOTDIR);
             }
         }
 
-        let mut flags = if path.base.is_empty() {
-            libc::AT_EMPTY_PATH
-        } else {
-            // SAFETY: After this point we are not permitted to resolve
-            // symbolic links any longer or else we risk TOCTOU.
-            libc::AT_SYMLINK_NOFOLLOW
-        };
+        // We use MUST_PATH, dir refers to the file.
+        assert!(
+            path.base.is_empty(),
+            "BUG: MUST_PATH returned a directory for stat, report a bug!"
+        );
+        let fd = path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?;
+        let mut flags = libc::AT_EMPTY_PATH;
 
         #[allow(clippy::cast_possible_truncation)]
         if arg_stat == 4 {
@@ -366,12 +361,7 @@ fn syscall_stat_handler(
             }
 
             // Note, unlike statfs, stat does not EINTR.
-            let mut statx = statx(
-                path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?,
-                path.base,
-                flags,
-                mask,
-            )?;
+            let mut statx = statx(fd, "", flags, mask)?;
 
             // SAFETY: Check if the file is a sidechannel device and
             // update its access and modification times to match the
@@ -419,7 +409,7 @@ fn syscall_stat_handler(
 
             // SAFETY: In libc we trust.
             // Note, unlike statfs, stat does not EINTR.
-            let mut stat = fstatat64(path.dir.as_ref().map(|fd| fd.as_raw_fd()), path.base, flags)?;
+            let mut stat = fstatat64(Some(fd.as_raw_fd()), "", flags)?;
 
             // SAFETY: Check if the file is a sidechannel device and
             // update its access and modification times to match the
