@@ -3,6 +3,7 @@
 #[cfg(test)]
 use std::convert::TryInto;
 
+use bitflags::bitflags;
 #[cfg(test)]
 use strum::{EnumCount, IntoEnumIterator};
 #[cfg(test)]
@@ -66,6 +67,9 @@ pub enum ABI {
     /// Sixth Landlock ABI, introduced with
     /// [Linux 6.12](https://git.kernel.org/stable/c/e1b061b444fb01c237838f0d8238653afe6a8094).
     V6 = 6,
+    /// Seventh Landlock ABI, introduced with
+    /// [Linux 6.15](https://git.kernel.org/stable/c/72885116069abdd05c245707c3989fc605632970).
+    V7 = 7,
 }
 
 impl ABI {
@@ -102,8 +106,9 @@ impl From<i32> for ABI {
             3 => ABI::V3,
             4 => ABI::V4,
             5 => ABI::V5,
+            6 => ABI::V6,
             // Returns the greatest known ABI.
-            _ => ABI::V6,
+            _ => ABI::V7,
         }
     }
 }
@@ -677,3 +682,63 @@ where
         }
     }
 }
+
+bitflags! {
+    /// Kernel "errata" reported by `landlock_create_ruleset(NULL, 0, ERRATA)`.
+    ///
+    /// The errata interface returns a bitmask of user-visible fixes that may
+    /// relax or clarify behavior. Bits are **1-based** in kernel docs, i.e.,
+    /// "Erratum 1" corresponds to bit 0 in this mask.
+    ///
+    /// Known errata:
+    ///
+    /// - **TCP socket identification (Erratum 1, tied to ABI v4):**
+    ///   Fixes non-TCP stream protocols (e.g., SMC/MPTCP/SCTP) being gated by
+    ///   TCP rights during `bind(2)`/`connect(2)`. With this bit set, only TCP
+    ///   sockets are governed by TCP access rights.
+    /// - **Scoped signal handling (Erratum 2, tied to ABI v6):**
+    ///   Always allow signals between threads of the same process (same
+    ///   thread-group), even with signal scoping enabled. This matches how
+    ///   user-space updates credentials across threads and mirrors ptrace rules.
+    ///
+    /// Notes:
+    /// - The mask is **runtime** feature detection and may be backported.
+    /// - Absence of a bit means the fix is not present on this kernel.
+    /// - This is orthogonal to the Landlock ABI version probe.
+    ///
+    /// See: `landlock_create_ruleset(2)` and `landlock(7)`.
+    #[derive(Clone, Copy, Debug, Default, Eq, PartialEq, Hash)]
+    pub struct Errata: u32 {
+        /// Erratum 1: TCP socket identification (impacts ABI v4).
+        ///
+        /// Ensures only TCP sockets are controlled by TCP access rights; other
+        /// stream protocols are unaffected by TCP rules.
+        const TCP_SOCKET_IDENTIFICATION = 1 << 0;
+
+        /// Erratum 2: Scoped signal handling (impacts ABI v6).
+        ///
+        /// Always allow signals between threads of the same process,
+        /// independent of Landlock signal scoping.
+        const SCOPED_SIGNAL_SAME_TGID   = 1 << 1;
+    }
+}
+
+impl Errata {
+    /// Query the running kernel for enabled errata and return them as flags.
+    ///
+    /// On kernels without Landlock or without the errata probe, this returns
+    /// `Ok(ErrataFlags::empty())`.
+    ///
+    /// Unknown errata flags are retained.
+    pub fn query() -> Self {
+        match unsafe {
+            // Landlock ABI version starts at 1 but errno is only set for negative values.
+            uapi::landlock_create_ruleset(std::ptr::null(), 0, uapi::LANDLOCK_CREATE_RULESET_ERRATA)
+        } {
+            -1 => Self::empty(),
+            n => u32::try_from(n)
+                .map(Self::from_bits_retain)
+                .unwrap_or(Self::empty()),
+        }
+    }
+}
