@@ -12,16 +12,17 @@ use std::{
     time::{SystemTime, UNIX_EPOCH},
 };
 
+use btoi::btoi;
 use data_encoding::HEXLOWER;
 use nix::{
     errno::Errno,
-    unistd::{fdatasync, write, Pid, Uid},
+    unistd::{write, Pid, Uid},
 };
 use serde_json::{Map, Value};
 
 use crate::{
     config::*,
-    fs::{lock_fd, retry_on_eintr, unlock_fd},
+    fs::{is_active_fd, lock_fd, unlock_fd},
     proc::*,
     syslog::LogLevel,
 };
@@ -226,13 +227,7 @@ impl Write for LockedWriter<'_> {
     }
 
     fn flush(&mut self) -> std::io::Result<()> {
-        // SAFETY:
-        // 1. Retry on EINTR.
-        // 2. Ignore EINVAL which indicates special file.
-        match retry_on_eintr(|| fdatasync(self.fd)) {
-            Ok(_) | Err(Errno::EINVAL) => Ok(()),
-            Err(e) => Err(std::io::Error::from_raw_os_error(e as i32)),
-        }
+        Ok(())
     }
 }
 
@@ -252,15 +247,23 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
         default_level
     };
 
-    // Determine if syslog host forwarding is on or off
-    let host_syslog = std::env::var_os(ENV_NO_SYSLOG).is_none();
-
     // Determine the main FD for logging:
-    // By default, we use stderr. If ENV_LOG_FD is set, we parse it.
+    // By default, we use stderr.
+    // If ENV_LOG_FD is set, we parse it:
+    // 1. negative fd is ok as a shorthand to disable logging.
+    // 2. positive fd must be a valid fd or we bail with EBADF.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => default_log_fd,
         Some(val) => {
-            Some(btoi::btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?)
+            let fd = btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?;
+            if fd >= 0 {
+                // SAFETY: fd only used after validation.
+                let fd = unsafe { BorrowedFd::borrow_raw(fd) };
+                if !is_active_fd(fd) {
+                    return Err(Errno::EBADF);
+                }
+            }
+            Some(fd)
         }
     };
     if let Some(fd) = fd {
@@ -270,10 +273,11 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
     // Decide if we want a TTY-like console.
     let mut tty = std::env::var_os(ENV_FORCE_TTY).is_some();
     if !tty {
-        if fd == Some(libc::STDERR_FILENO) && std::env::var_os(ENV_QUIET_TTY).is_none() {
-            // check if we have a TTY
-            tty = nix::unistd::isatty(std::io::stdin()).unwrap_or(false)
-                && nix::unistd::isatty(std::io::stderr()).unwrap_or(false);
+        if std::env::var_os(ENV_QUIET_TTY).is_none() {
+            let fd = fd.unwrap_or(libc::STDERR_FILENO);
+            // Check if log FD is a TTY.
+            // SAFETY: In libc we trust.
+            tty = unsafe { libc::isatty(fd) } == 1;
         } else {
             tty = false;
         }
@@ -298,7 +302,7 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
             .or(Err(Errno::EINVAL))?;
         use_stack = false;
     }
-    init_global_syslog(logbuflen, fd, level, host_syslog, use_stack)?;
+    init_global_syslog(logbuflen, fd, level, use_stack)?;
 
     // Finally let's make some noise!
     info!("ctx": "init", "op": "sing", "chapter": 24,
@@ -324,24 +328,34 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
         default_level
     };
 
-    // We'll turn OFF host syslog, always.
-    let host_syslog = false;
-
     // Determine the main FD for logging:
-    // By default, we use stderr. If ENV_LOG_FD is set, we parse it.
+    // By default, we use stderr.
+    // If ENV_LOG_FD is set, we parse it:
+    // 1. negative fd is ok as a shorthand to disable logging.
+    // 2. positive fd must be a valid fd or we bail with EBADF.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => libc::STDERR_FILENO,
-        Some(val) => btoi::btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?,
+        Some(val) => {
+            let fd = btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?;
+            if fd >= 0 {
+                // SAFETY: fd only used after validation.
+                let fd = unsafe { BorrowedFd::borrow_raw(fd) };
+                if !is_active_fd(fd) {
+                    return Err(Errno::EBADF);
+                }
+            }
+            fd
+        }
     };
     LOG_FD.store(fd, std::sync::atomic::Ordering::Relaxed);
 
     // Decide if we want a TTY-like console.
     let mut tty = std::env::var_os(ENV_FORCE_TTY).is_some();
     if !tty {
-        if fd == libc::STDERR_FILENO && std::env::var_os(ENV_QUIET_TTY).is_none() {
-            // check if we have a TTY
-            tty = nix::unistd::isatty(std::io::stdin()).unwrap_or(false)
-                && nix::unistd::isatty(std::io::stderr()).unwrap_or(false);
+        if std::env::var_os(ENV_QUIET_TTY).is_none() {
+            // Check if log FD is a TTY.
+            // SAFETY: In libc we trust.
+            tty = unsafe { libc::isatty(fd) } == 1;
         } else {
             tty = false;
         }
@@ -350,20 +364,8 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
     // Record TTY information to an atomic for easy access.
     LOG_TTY.store(tty, std::sync::atomic::Ordering::Relaxed);
 
-    // We'll do the same FD detection.
-    let mut tty = std::env::var_os(ENV_FORCE_TTY).is_some();
-    if !tty {
-        if std::env::var_os(ENV_QUIET_TTY).is_none() {
-            tty = nix::unistd::isatty(std::io::stdin()).unwrap_or(false)
-                && nix::unistd::isatty(std::io::stderr()).unwrap_or(false);
-        } else {
-            tty = false;
-        }
-    }
-    LOG_TTY.store(tty, std::sync::atomic::Ordering::Relaxed);
-
     // Create a global syslog with ring.
-    init_global_syslog(0, Some(fd), level, host_syslog, true)?;
+    init_global_syslog(0, Some(fd), level, true)?;
 
     if let Some(sys) = global_syslog() {
         // Now lock it immediately,
@@ -460,16 +462,41 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
 /// Initializes Syslog global object, reading environment variables.
 #[cfg(not(feature = "log"))]
 #[inline(always)]
+#[allow(clippy::cognitive_complexity)]
 pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Result<(), Errno> {
     use std::os::unix::ffi::OsStrExt;
 
+    // Determine the main FD for logging:
+    // By default, we use stderr.
+    // If ENV_LOG_FD is set, we parse it:
+    // 1. negative fd is ok as a shorthand to disable logging.
+    // 2. positive fd must be a valid fd or we bail with EBADF.
+    let fd = match std::env::var_os(ENV_LOG_FD) {
+        None => default_log_fd,
+        Some(val) => {
+            let fd = btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?;
+            if fd >= 0 {
+                // SAFETY: fd only used after validation.
+                let fd = unsafe { BorrowedFd::borrow_raw(fd) };
+                if !is_active_fd(fd) {
+                    return Err(Errno::EBADF);
+                }
+            }
+            Some(fd)
+        }
+    };
+    if let Some(fd) = fd {
+        LOG_FD.store(fd, std::sync::atomic::Ordering::Relaxed);
+    }
+
     // Decide if we want a TTY-like console.
     let mut tty = std::env::var_os(ENV_FORCE_TTY).is_some();
     if !tty {
-        if std::env::var_os(ENV_LOG_FD).is_none() && std::env::var_os(ENV_QUIET_TTY).is_none() {
-            // check if we have a TTY
-            tty = nix::unistd::isatty(std::io::stdin()).unwrap_or(false)
-                && nix::unistd::isatty(std::io::stderr()).unwrap_or(false);
+        if std::env::var_os(ENV_QUIET_TTY).is_none() {
+            let fd = fd.unwrap_or(libc::STDERR_FILENO);
+            // Check if log FD is a TTY.
+            // SAFETY: In libc we trust.
+            tty = unsafe { libc::isatty(fd) } == 1;
         } else {
             tty = false;
         }
@@ -479,11 +506,22 @@ pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resu
     LOG_TTY.store(tty, std::sync::atomic::Ordering::Relaxed);
 
     // Determine the main FD for logging:
-    // By default, we use stderr. If ENV_LOG_FD is set, we parse it.
+    // By default, we use stderr.
+    // If ENV_LOG_FD is set, we parse it:
+    // 1. negative fd is ok as a shorthand to disable logging.
+    // 2. positive fd must be a valid fd or we bail with EBADF.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => default_log_fd,
         Some(val) => {
-            Some(btoi::btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?)
+            let fd = btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?;
+            if fd >= 0 {
+                // SAFETY: fd only used after validation.
+                let fd = unsafe { BorrowedFd::borrow_raw(fd) };
+                if !is_active_fd(fd) {
+                    return Err(Errno::EBADF);
+                }
+            }
+            Some(fd)
         }
     };
     if let Some(fd) = fd {
@@ -508,9 +546,9 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
 #[cfg(not(feature = "log"))]
 #[allow(clippy::cognitive_complexity)]
 pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String, Value>) {
-    // For "higher" severities, we add more contextual fields, and honour TTY.
+    // For "higher" severities, we add more contextual fields.
     let add_context = level.as_u8() <= crate::syslog::LogLevel::Notice.as_u8();
-    let tty = add_context && LOG_TTY.load(std::sync::atomic::Ordering::Relaxed);
+    let tty = LOG_TTY.load(std::sync::atomic::Ordering::Relaxed);
 
     // If there's a "pid", we might add cmd/cwd, etc.
     // We remove and readd to reorder for better visibility.
