@@ -11,8 +11,11 @@
 
 //! syd(2) api parsers and utility functions.
 
-use std::str::FromStr;
+use std::{ops::RangeInclusive, str::FromStr};
 
+use btoi::{btoi, btoi_radix};
+use fixedbitset::FixedBitSet;
+use memchr::arch::all::is_prefix;
 use nix::{errno::Errno, mount::MsFlags};
 use nom::{
     branch::alt,
@@ -25,19 +28,14 @@ use nom::{
 };
 
 use crate::{
+    confine::SydMsFlags,
+    hash::SydHashSet,
+    landlock::{AccessFs, AccessNet},
+    landlock_policy::LandlockPolicy,
     path::XPathBuf,
     sandbox::{Action, BindMount, Capability},
-    SydMsFlags,
 };
 
-// Valid rights for "allow/lock" commands.
-//
-// Note, this list must be sorted because it's binary searched.
-const LANDLOCK_RIGHTS: &[&str] = &[
-    "bind", "connect", "create", "delete", "exec", "ioctl", "mkdev", "mkdir", "mkfifo", "read",
-    "readdir", "rename", "rmdir", "symlink", "truncate", "write",
-];
-
 // Valid Netlink families.
 //
 // Note, this list must be sorted because it's binary searched.
@@ -101,28 +99,59 @@ pub(crate) struct SetIdCmd {
     pub(crate) dst: Option<String>,
 }
 
-/// Represents the access filter: either "all" or a comma-separated list of rights.
-#[derive(Debug, PartialEq, Eq)]
-pub enum LandlockFilter {
-    /// All categories
-    All,
-    /// Selected categories
-    Many(Vec<String>),
+/// Network port range
+pub type PortRange = RangeInclusive<u16>;
+
+/// Set of paths
+pub type PathSet = SydHashSet<XPathBuf>;
+
+/// Fixed bit set of port ranges
+pub type PortSet = FixedBitSet;
+
+/// landlock(7) access control rule
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub enum LandlockRule {
+    /// landlock(7) filesystem rule
+    Fs((AccessFs, String)),
+    /// landlock(7) network fule
+    Net((AccessNet, PortRange)),
+}
+
+/// Array of Landlock access control rules
+pub type LandlockFilter = Vec<LandlockRule>;
+
+/// Represents a Landlock rule operation.
+#[derive(Debug, Copy, Clone, PartialEq, Eq)]
+pub enum LandlockOp {
+    /// Add operation
+    Add,
+    /// Remove-all operation
+    Rem,
+}
+
+impl TryFrom<char> for LandlockOp {
+    type Error = Errno;
+
+    fn try_from(c: char) -> Result<Self, Self::Error> {
+        match c {
+            '+' => Ok(Self::Add),
+            '-' | '^' => Ok(Self::Rem),
+            _ => Err(Errno::EINVAL),
+        }
+    }
 }
 
 /// Parsed "allow/lock" command.
 #[derive(Debug, PartialEq, Eq)]
 pub struct LandlockCmd {
-    /// Access filter (all or specific rights)
+    /// Access filter
     pub filter: LandlockFilter,
-    /// One of '+', '-', '^'
-    pub op: char,
-    /// Argument (path or port range)
-    pub arg: String,
+    /// Operation: add or remove.
+    pub op: LandlockOp,
 }
 
 // Pattern for seccomp rule: either a filesystem path or an IP-based address.
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub(crate) enum ScmpPattern {
     Path(String),
     Addr(String),
@@ -130,7 +159,7 @@ pub(crate) enum ScmpPattern {
 }
 
 // Parsed seccomp rule command.
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub(crate) struct ScmpCmd {
     pub(crate) action: Action,
     pub(crate) filter: Capability,
@@ -139,7 +168,7 @@ pub(crate) struct ScmpCmd {
 }
 
 // Operation for Netlink families.
-#[derive(Debug, PartialEq, Eq)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub(crate) enum NetlinkOp {
     Clear,
     Add(Vec<String>),
@@ -147,7 +176,7 @@ pub(crate) enum NetlinkOp {
 }
 
 // Parsed Netlink command.
-#[derive(Debug, PartialEq, Eq)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub(crate) struct NetlinkCmd {
     pub(crate) op: NetlinkOp,
 }
@@ -402,6 +431,32 @@ pub(crate) fn parse_setid_cmd(input: &str) -> Result<SetIdCmd, Errno> {
     }
 }
 
+/// Parse a port range which is either a single port
+/// or a closed range in format "port1-port2".
+pub fn parse_port_range(input: &str) -> Result<PortRange, Errno> {
+    let mut split = input.splitn(2, '-');
+
+    let port0 = split.next().ok_or(Errno::EINVAL)?;
+    let port0 = port0.parse::<u16>().or(Err(Errno::EINVAL))?;
+
+    let ports = if let Some(port1) = split.next() {
+        let port1 = port1.parse::<u16>().or(Err(Errno::EINVAL))?;
+        if port1 >= port0 {
+            port0..=port1
+        } else {
+            port1..=port0
+        }
+    } else {
+        port0..=port0
+    };
+
+    if ports.is_empty() {
+        return Err(Errno::EINVAL);
+    }
+
+    Ok(ports)
+}
+
 /// Parse an "allow/lock" command: "allow/lock/<access_list><op><arg>"
 /// Returns `LandlockCmd` or `Errno::EINVAL`.
 pub fn parse_landlock_cmd(input: &str) -> Result<LandlockCmd, Errno> {
@@ -421,37 +476,51 @@ pub fn parse_landlock_cmd(input: &str) -> Result<LandlockCmd, Errno> {
 
     match inner(input).finish() {
         Ok(("", (access, op, arg))) => {
-            // Determine filter.
-            let filter = if access == "all" {
-                LandlockFilter::All
-            } else {
-                let mut rights: Vec<String> = vec![];
+            // Determine operation and access rights.
+            let op = LandlockOp::try_from(op)?;
+            let (mut access_fs, access_net) = LandlockPolicy::access(access)?;
 
-                // Split on commas and validate each right.
-                for right in access.split(',') {
-                    // Reject empty rights.
-                    if right.is_empty() {
-                        return Err(Errno::EINVAL);
-                    }
-
-                    // Validate right using binary search.
-                    if let Ok(idx) = LANDLOCK_RIGHTS.binary_search(&right) {
-                        rights.push(LANDLOCK_RIGHTS[idx].to_string());
-                    } else {
-                        return Err(Errno::EINVAL);
-                    }
+            // Validate access rights.
+            let ports = parse_port_range(arg).ok();
+            if access_net.contains(AccessNet::ConnectTcp) {
+                // connect requires a port-range.
+                if ports.is_none() {
+                    return Err(Errno::EINVAL);
                 }
 
-                LandlockFilter::Many(rights)
-            };
+                // bind implies BindTcp+MakeSock.
+                if access_fs == AccessFs::MakeSock {
+                    // bind,connect
+                    access_fs = AccessFs::EMPTY;
+                } else if !access_fs.is_empty() {
+                    // connect,<filesystem-right>
+                    return Err(Errno::EINVAL);
+                }
+            }
 
-            // Argument must be non-empty, no further validation here.
-            if arg.is_empty() {
+            let mut filter = LandlockFilter::new();
+            if access_net == AccessNet::BindTcp && access_fs == AccessFs::MakeSock {
+                // Require absolute pathnames for UNIX domain sockets.
+                // Allow environment variables as well which will be treated as paths.
+                // This way passing a relative UNIX domain socket path is still possible.
+                let c = arg.chars().next().ok_or(Errno::EINVAL)?;
+                if matches!(c, '/' | '$') {
+                    filter.push(LandlockRule::Fs((access_fs, arg.into())));
+                } else {
+                    let ports = ports.ok_or(Errno::EINVAL)?;
+                    filter.push(LandlockRule::Net((access_net, ports)));
+                }
+            } else if !access_fs.is_empty() {
+                filter.push(LandlockRule::Fs((access_fs, arg.into())));
+            } else if access_net.contains(AccessNet::ConnectTcp) {
+                let ports = ports.ok_or(Errno::EINVAL)?;
+                filter.push(LandlockRule::Net((access_net, ports)));
+            } else {
+                eprintln!("LO:4");
                 return Err(Errno::EINVAL);
             }
-            let arg = arg.to_string();
 
-            Ok(LandlockCmd { arg, filter, op })
+            Ok(LandlockCmd { filter, op })
         }
         _ => Err(Errno::EINVAL),
     }
@@ -676,6 +745,38 @@ fn host_parser(input: &str) -> IResult<&str, &str> {
     recognize(all_consuming(alias_tuple)).parse(input)
 }
 
+/// Converts a string representation of a number into a `u64` value.
+///
+/// The string can be in hexadecimal (prefixed with "0x"), octal
+/// (prefixed with "0o"), or decimal format. If the conversion fails, it
+/// returns an `Errno::EINVAL` error.
+pub fn str2u64(value: &[u8]) -> Result<u64, Errno> {
+    if is_prefix(value, b"0x") || is_prefix(value, b"0X") {
+        btoi_radix::<u64>(&value[2..], 16)
+    } else if is_prefix(value, b"0o") || is_prefix(value, b"0O") {
+        btoi_radix::<u64>(&value[2..], 8)
+    } else {
+        btoi::<u64>(value)
+    }
+    .or(Err(Errno::EINVAL))
+}
+
+/// Converts a string representation of a number into a `u32` value.
+///
+/// The string can be in hexadecimal (prefixed with "0x"), octal
+/// (prefixed with "0o"), or decimal format. If the conversion fails, it
+/// returns an `Errno::EINVAL` error.
+pub fn str2u32(value: &[u8]) -> Result<u32, Errno> {
+    if is_prefix(value, b"0x") || is_prefix(value, b"0X") {
+        btoi_radix::<u32>(&value[2..], 16)
+    } else if is_prefix(value, b"0o") || is_prefix(value, b"0O") {
+        btoi_radix::<u32>(&value[2..], 8)
+    } else {
+        btoi::<u32>(value)
+    }
+    .or(Err(Errno::EINVAL))
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -936,52 +1037,327 @@ mod tests {
     #[test]
     fn test_parse_landlock_parse_all_plus_path() {
         let cmd = parse_landlock_cmd("allow/lock/all+/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
         assert_eq!(
-            cmd,
-            LandlockCmd {
-                filter: LandlockFilter::All,
-                op: '+',
-                arg: "/trusted".into(),
-            }
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((LandlockPolicy::access_fs_from_set("all"), "/trusted".into(),))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_all_minus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/all-/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((LandlockPolicy::access_fs_from_set("all"), "/trusted".into(),))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_all_caret_path() {
+        let cmd = parse_landlock_cmd("allow/lock/all^/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((LandlockPolicy::access_fs_from_set("all"), "/trusted".into(),))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_rpath_plus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/rpath+/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("rpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_rpath_minus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/rpath-/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("rpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_rpath_caret_path() {
+        let cmd = parse_landlock_cmd("allow/lock/rpath^/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("rpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_wpath_plus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/wpath+/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("wpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_wpath_minus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/wpath-/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("wpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_wpath_caret_path() {
+        let cmd = parse_landlock_cmd("allow/lock/wpath^/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("wpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_cpath_plus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/cpath+/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("cpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_cpath_minus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/cpath-/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("cpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_cpath_caret_path() {
+        let cmd = parse_landlock_cmd("allow/lock/cpath^/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("cpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_dpath_plus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/dpath+/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("dpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_dpath_minus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/dpath-/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("dpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_dpath_caret_path() {
+        let cmd = parse_landlock_cmd("allow/lock/dpath^/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("dpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_spath_plus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/spath+/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("spath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_spath_minus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/spath-/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("spath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_spath_caret_path() {
+        let cmd = parse_landlock_cmd("allow/lock/spath^/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("spath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_tpath_plus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/tpath+/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("tpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_tpath_minus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/tpath-/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("tpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_tpath_caret_path() {
+        let cmd = parse_landlock_cmd("allow/lock/tpath^/trusted").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                LandlockPolicy::access_fs_from_set("tpath"),
+                "/trusted".into(),
+            ))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_net_inet_plus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/inet+1024-65535").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Net((LandlockPolicy::access_net_from_set("inet"), 1024..=65535,))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_net_inet_minus_path() {
+        let cmd = parse_landlock_cmd("allow/lock/inet-1024-65535").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Net((LandlockPolicy::access_net_from_set("inet"), 1024..=65535,))
+        );
+    }
+
+    #[test]
+    fn test_parse_landlock_parse_net_inet_caret_path() {
+        let cmd = parse_landlock_cmd("allow/lock/inet^1024-65535").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
+        assert_eq!(
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Net((LandlockPolicy::access_net_from_set("inet"), 1024..=65535,))
         );
     }
 
     #[test]
     fn test_parse_landlock_parse_many_fs_rights_minus() {
         let cmd = parse_landlock_cmd("allow/lock/read,write,exec-/var/log").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
         assert_eq!(
-            cmd,
-            LandlockCmd {
-                filter: LandlockFilter::Many(vec!["read".into(), "write".into(), "exec".into()]),
-                op: '-',
-                arg: "/var/log".into(),
-            }
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((
+                AccessFs::ReadFile | AccessFs::WriteFile | AccessFs::Execute,
+                "/var/log".into(),
+            ))
         );
     }
 
     #[test]
     fn test_parse_landlock_parse_many_net_rights_caret() {
         let cmd = parse_landlock_cmd("allow/lock/bind,connect^1000-2000").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Rem);
         assert_eq!(
-            cmd,
-            LandlockCmd {
-                filter: LandlockFilter::Many(vec!["bind".into(), "connect".into()]),
-                op: '^',
-                arg: "1000-2000".into(),
-            }
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Net((AccessNet::BindTcp | AccessNet::ConnectTcp, 1000..=2000,))
         );
     }
 
     #[test]
     fn test_parse_landlock_parse_single_right_write_plus() {
         let cmd = parse_landlock_cmd("allow/lock/write+tmp").unwrap();
+        assert_eq!(cmd.op, LandlockOp::Add);
         assert_eq!(
-            cmd,
-            LandlockCmd {
-                filter: LandlockFilter::Many(vec!["write".into()]),
-                op: '+',
-                arg: "tmp".into(),
-            }
+            cmd.filter.first().cloned().unwrap(),
+            LandlockRule::Fs((AccessFs::WriteFile, "tmp".into(),))
         );
     }
 
