@@ -25,10 +25,10 @@
 #![deny(unsafe_code)]
 
 use std::{
-    collections::{HashMap, HashSet},
-    ffi::{CStr, OsStr},
+    ffi::{CStr, OsStr, OsString},
     fs::File,
-    io::{BufReader, IoSlice, IoSliceMut, Read},
+    io::{BufReader, IoSlice, IoSliceMut, Read, Seek, SeekFrom},
+    num::NonZeroUsize,
     ops::Range,
     os::{
         fd::{AsFd, AsRawFd, OwnedFd, RawFd},
@@ -40,41 +40,41 @@ use std::{
 use bitflags::bitflags;
 use btoi::{btoi, btoi_radix};
 use indexmap::IndexSet;
-use memchr::{
-    arch::all::{is_equal, is_prefix},
-    memchr,
-};
 use nix::{
     errno::Errno,
-    fcntl::{readlinkat, OFlag},
+    fcntl::{open, readlinkat, OFlag},
     libc::pid_t,
     sys::{
         stat::Mode,
         sysinfo::sysinfo,
         uio::{process_vm_readv, process_vm_writev, RemoteIoVec},
     },
-    unistd::{getpid, lseek64, read, Gid, Pid, Uid, Whence},
+    unistd::{getpid, lseek64, read, write, Gid, Pid, Uid, Whence},
 };
-use procfs::{
-    process::{LimitValue, MMPermissions, MMapPath, MemoryMaps},
-    FromBufRead,
+use once_cell::sync::Lazy;
+use procfs_core::{
+    net::UnixNetEntries,
+    process::{LimitValue, MMPermissions, MMapPath, MemoryMaps, Namespace, Namespaces},
+    CryptoTable, FromBufRead,
 };
 
 use crate::{
-    compat::{getdents64, trim_ascii},
+    compat::{getdents64, statx, trim_ascii, STATX_INO},
     config::*,
+    confine::SydMemoryMap,
     cookie::{PROCMAP_QUERY_COOKIE_ARG3, PROCMAP_QUERY_COOKIE_ARG4, PROCMAP_QUERY_COOKIE_ARG5},
     elf::ElfType,
     err::err2no,
     fs::{is_open_fd, parse_fd, randint, safe_open, AT_BADFD},
-    hash::SydRandomState,
+    hash::{SydHashMap, SydHashSet},
     parsers::{
         map_result,
         proc::{parse_max_open_files, parse_stat, parse_statm, parse_status},
         read_to_end,
     },
     path::{XPathBuf, PATH_MAX},
-    SydMemoryMap, SydSigSet, XPath,
+    sigset::SydSigSet,
+    XPath,
 };
 
 bitflags! {
@@ -393,20 +393,6 @@ const LOW_DENSITY_NAMES: &[&str] = &[
     "SC1", "SC2", "SC3", "MAX0", "MAX1", "MAX2", "MAX3",
 ];
 
-#[allow(clippy::disallowed_methods)]
-#[inline(always)]
-fn bytes_to_pid(bytes: &[u8]) -> Result<Pid, Errno> {
-    btoi::<pid_t>(bytes)
-        .map(Pid::from_raw)
-        .or(Err(Errno::EINVAL))
-}
-
-#[allow(clippy::disallowed_methods)]
-#[inline(always)]
-fn bytes_to_fd(bytes: &[u8]) -> Result<RawFd, Errno> {
-    btoi::<RawFd>(bytes).or(Err(Errno::EINVAL))
-}
-
 fn read_usize_from_ne_bytes(bytes: &[u8], ptr_size: usize) -> Result<usize, Errno> {
     match ptr_size {
         4 => {
@@ -558,7 +544,7 @@ pub fn proc_stack_pointer(pid: Pid) -> Result<u64, Errno> {
 }
 
 /// Get the auxiliary vector of the given `Pid`.
-pub fn proc_auxv(pid: Pid) -> Result<HashMap<u64, u64, SydRandomState>, Errno> {
+pub fn proc_auxv(pid: Pid) -> Result<SydHashMap<u64, u64>, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"auxv");
 
@@ -566,7 +552,7 @@ pub fn proc_auxv(pid: Pid) -> Result<HashMap<u64, u64, SydRandomState>, Errno> {
         .map(File::from)
         .or(Err(Errno::ESRCH))?;
 
-    let mut map = HashMap::default();
+    let mut map = SydHashMap::default();
 
     let mut buf = Vec::new();
     let bytes_read = file.read_to_end(&mut buf).map_err(|err| err2no(&err))?;
@@ -1048,102 +1034,6 @@ pub fn proc_task_limit(pid: Pid, max: u64) -> Result<bool, Errno> {
     Ok(false)
 }
 
-const FD: &[u8] = b"/fd/";
-const PROC: &[u8] = b"/proc/";
-const TASK: &[u8] = b"/task/";
-
-/// 0. Assumes the given path is normalized.
-/// 1. The path must start with /proc.
-/// 2. The second component must be a numeric PID equal to the given PID.
-/// 3. An optional task/[TID] part, where [TID] is a numeric PID.
-/// 4. Finally, it checks for an fd component followed by a numeric file descriptor.
-///    Note, This function does not allocate.
-///    Note, use negated PID if you only need validation and not the exact FD value.
-///
-/// SAFETY: If `restrict_magiclinks` is `true`, this function returns
-/// Err(Errno::EACCES) if PID is present but not equal to given PID.
-#[allow(clippy::cognitive_complexity)]
-#[allow(clippy::type_complexity)]
-pub fn proc_fd(
-    pid: Pid,
-    path: &XPath,
-    restrict_magiclinks: bool,
-) -> Result<Option<(RawFd, Pid)>, Errno> {
-    let path = path.as_bytes();
-    if !is_prefix(path, PROC) {
-        // Not a /proc path.
-        return Ok(None);
-    }
-
-    let path_without_proc = &path[PROC.len()..];
-    let next_slash_index = memchr(b'/', path_without_proc).unwrap_or(path_without_proc.len());
-    let pid_section = &path_without_proc[..next_slash_index];
-    if pid_section
-        .first()
-        .map(|c| !c.is_ascii_digit())
-        .unwrap_or(true)
-    {
-        // Skip /proc/not-a-pid paths.
-        return Ok(None);
-    }
-
-    let mut pid = if restrict_magiclinks {
-        if pid != bytes_to_pid(pid_section)? {
-            // PID mismatch detected!
-            //
-            // SAFETY: Note, ideally we want to return
-            // ENOENT here for stealth, however this
-            // confuses programs such as pipewire when
-            // they're checking for flatpak support.
-            // Check for pw_check_flatpak() function
-            // in pipewire source code for more information.
-            return Err(Errno::EACCES);
-        }
-        pid
-    } else {
-        bytes_to_pid(pid_section)?
-    };
-
-    let after_pid_section = &path_without_proc[next_slash_index..];
-    let start_of_interesting_part = if after_pid_section.starts_with(TASK) {
-        let after_task_section = &after_pid_section[TASK.len()..];
-        match memchr(b'/', after_task_section) {
-            Some(idx) => {
-                pid = bytes_to_pid(&after_task_section[..idx])?;
-                idx
-            }
-            None => after_task_section.len(),
-        }
-        .saturating_add(TASK.len())
-    } else {
-        0
-    };
-
-    let remaining_path = &after_pid_section[start_of_interesting_part..];
-    if is_prefix(remaining_path, FD) {
-        // Check for valid file descriptor number after /fd/
-        let fd_section = &remaining_path[FD.len()..];
-
-        Ok(if fd_section.iter().all(|c| c.is_ascii_digit()) {
-            Some((bytes_to_fd(fd_section)?, pid))
-        } else {
-            None
-        })
-    } else if is_equal(remaining_path, b"/cwd") {
-        // Magic CWD.
-        Ok(Some((libc::AT_FDCWD, pid)))
-    } else if is_equal(remaining_path, b"/root") {
-        // Magic ROOT.
-        Ok(Some((-1, pid)))
-    } else if is_equal(remaining_path, b"/exe") {
-        // Magic EXE.
-        Ok(Some((-2, pid)))
-    } else {
-        // Not a magic symlink.
-        Ok(None)
-    }
-}
-
 /// Checks whether process memory usage is within the give maximum.
 ///
 /// This function uses the `procfs` crate to obtain detailed memory maps
@@ -1312,14 +1202,14 @@ pub struct Vma {
     ptr: NonNull<libc::c_void>,
 
     // VMA length
-    len: usize,
+    len: NonZeroUsize,
 
     // VMA flags
     flags: ProcmapQueryFlags,
 }
 
 impl Vma {
-    fn new(ptr: NonNull<libc::c_void>, len: usize, flags: ProcmapQueryFlags) -> Self {
+    fn new(ptr: NonNull<libc::c_void>, len: NonZeroUsize, flags: ProcmapQueryFlags) -> Self {
         Self { ptr, len, flags }
     }
 
@@ -1329,7 +1219,7 @@ impl Vma {
     }
 
     /// Return VMA length.
-    pub fn len(&self) -> usize {
+    pub fn len(&self) -> NonZeroUsize {
         self.len
     }
 
@@ -1337,11 +1227,6 @@ impl Vma {
     pub fn flags(&self) -> ProcmapQueryFlags {
         self.flags
     }
-
-    /// Return true if VMA is empty.
-    pub fn is_empty(&self) -> bool {
-        self.len == 0
-    }
 }
 
 /// Get the VMA covering `addr` in process `pid`.
@@ -1366,9 +1251,11 @@ pub fn proc_get_vma(pid: Pid, addr: u64) -> Result<Vma, Errno> {
             let ptr = NonNull::new(start as *mut _).ok_or(Errno::EINVAL)?;
             let len = end
                 .checked_sub(start)
-                .ok_or(Errno::EINVAL)?
-                .try_into()
-                .or(Err(Errno::EINVAL))?;
+                .ok_or(Errno::EINVAL)
+                .map(usize::try_from)?
+                .or(Err(Errno::EINVAL))
+                .map(NonZeroUsize::new)?
+                .ok_or(Errno::EINVAL)?;
             return Ok(Vma::new(ptr, len, map.perms.into()));
         }
     }
@@ -1392,17 +1279,134 @@ fn procmap_query_get_vma(pid: Pid, addr: u64) -> Result<Vma, Errno> {
     let len = q
         .vma_end
         .checked_sub(q.vma_start)
-        .ok_or(Errno::EINVAL)?
-        .try_into()
-        .or(Err(Errno::EINVAL))?;
+        .ok_or(Errno::EINVAL)
+        .map(usize::try_from)?
+        .or(Err(Errno::EINVAL))
+        .map(NonZeroUsize::new)?
+        .ok_or(Errno::EINVAL)?;
     let flags = ProcmapQueryFlags::from_bits_truncate(q.vma_flags);
 
     Ok(Vma::new(ptr, len, flags))
 }
 
+/// Parses /proc/pid/environ and returns a HashMap.
+pub fn proc_environ(pid: Pid) -> Result<SydHashMap<OsString, OsString>, Errno> {
+    proc_environ_read(proc_environ_open(pid)?)
+}
+
+/// Open a FD to _proc_pid_environ_(5) and return an OwnedFd.
+pub fn proc_environ_open(pid: Pid) -> Result<OwnedFd, Errno> {
+    let mut path = XPathBuf::from_pid(pid);
+    path.push(b"environ");
+
+    safe_open(PROC_FILE(), &path, OFLAG_PROC).or(Err(Errno::ESRCH))
+}
+
+/// Parses /proc/pid/environ from given Fd and returns a HashMap.
+pub fn proc_environ_read(fd: OwnedFd) -> Result<SydHashMap<OsString, OsString>, Errno> {
+    let mut buf = Vec::new();
+    File::from(fd)
+        .read_to_end(&mut buf)
+        .map_err(|err| err2no(&err))?;
+
+    let mut map = SydHashMap::default();
+    for slice in buf.split(|b| *b == 0) {
+        // slice will be in the form key=var, so split on the first equals sign.
+        let mut split = slice.splitn(2, |b| *b == b'=');
+        if let (Some(k), Some(v)) = (split.next(), split.next()) {
+            map.insert(
+                OsStr::from_bytes(k).to_os_string(),
+                OsStr::from_bytes(v).to_os_string(),
+            );
+        };
+        //let env = OsStr::from_bytes(slice);
+    }
+
+    Ok(map)
+}
+
+/// Describes namespaces to which the process with the corresponding PID belongs.
+///
+/// The namespace type is the key for the HashMap, i.e 'net', 'user', etc.
+pub fn proc_namespaces(pid: Pid) -> Result<Namespaces, Errno> {
+    // Open /proc/<pid>/ns as a directory fd.
+    //
+    // Note, we do not use PROC_FILE() here,
+    // because currently the only consumer of
+    // this function is syd-run where preopening
+    // /proc is needless.
+    let mut ns = XPathBuf::from("/proc");
+    ns.push_pid(pid);
+    ns.push(b"ns");
+    #[allow(clippy::disallowed_methods)]
+    let dirfd = open(&ns, OFLAG_PROC | OFlag::O_DIRECTORY, Mode::empty())?;
+
+    // Read directory entries using getdents64 in a loop.
+    #[allow(clippy::disallowed_types)]
+    let mut namespaces = std::collections::HashMap::new();
+    loop {
+        let mut entries = match getdents64(&dirfd, DIRENT_BUF_SIZE) {
+            Ok(iter) => iter,
+            Err(Errno::ECANCELED) => break, // EOF or empty directory
+            Err(errno) => return Err(errno),
+        };
+
+        for entry in &mut entries {
+            if entry.is_dot() {
+                continue;
+            }
+
+            let (ino, dev) = statx(&dirfd, entry.as_xpath(), 0, STATX_INO).map(|stx| {
+                let ino = stx.stx_ino;
+                let dev = (u64::from(stx.stx_dev_major) << 32) | u64::from(stx.stx_dev_minor);
+                (ino, dev)
+            })?;
+
+            let ns_type = OsStr::from_bytes(entry.name_bytes()).to_os_string();
+            let mut ns_path = XPathBuf::from("/proc");
+            ns_path.push_pid(pid);
+            ns_path.push(b"ns");
+            ns_path.push(entry.name_bytes());
+
+            let ns = Namespace {
+                ns_type: ns_type.clone(),
+                path: ns_path.to_path_buf(),
+                identifier: ino,
+                device_id: dev,
+            };
+
+            if namespaces.insert(ns_type, ns).is_some() {
+                return Err(Errno::EEXIST);
+            }
+        }
+    }
+
+    Ok(Namespaces(namespaces))
+}
+
+/// Parses /proc/crypto and returns the table of crypto algorithms.
+pub fn proc_crypto() -> Result<CryptoTable, Errno> {
+    proc_crypto_read(proc_crypto_open()?)
+}
+
+/// Open a FD to _proc_crypto_(5) and return an OwnedFd.
+pub fn proc_crypto_open() -> Result<OwnedFd, Errno> {
+    safe_open(PROC_FILE(), XPath::from_bytes(b"crypto"), OFLAG_PROC)
+}
+
+/// Parses /proc/crypto from given Fd and returns the table of crypto algorithms.
+pub fn proc_crypto_read(fd: OwnedFd) -> Result<CryptoTable, Errno> {
+    CryptoTable::from_buf_read(BufReader::new(File::from(fd))).or(Err(Errno::EPERM))
+}
+
 /// Parses /proc/net/unix and returns the inodes of all UNIX domain sockets.
-pub fn proc_unix_get_inodes() -> Result<HashSet<u64>, Errno> {
-    Ok(procfs::net::unix()
+pub fn proc_unix_get_inodes() -> Result<SydHashSet<u64>, Errno> {
+    let path = XPath::from_bytes(b"net/unix");
+    let file = safe_open(PROC_FILE(), path, OFLAG_PROC)
+        .map(File::from)
+        .map(BufReader::new)?;
+    Ok(UnixNetEntries::from_buf_read(file)
+        .map(|e| e.0)
         .or(Err(Errno::EPERM))?
         .into_iter()
         .filter(|entry| {
@@ -1435,6 +1439,119 @@ pub fn proc_unix_get_inodes() -> Result<HashSet<u64>, Errno> {
 ///
 /// Returns `Ok(())` on success, or an `Err(Errno)` if an error occurs.
 pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
+    // This function requires CONFIG_CROSS_MEMORY_ATTACH,
+    // unless Syd is explicitly configured to fallback.
+    static FORCE_PROC: Lazy<bool> =
+        Lazy::new(|| std::env::var_os(ENV_NO_CROSS_MEMORY_ATTACH).is_some());
+    static PROC_MEM_F: Lazy<bool> =
+        Lazy::new(|| std::env::var_os(ENV_PROC_PID_MEM_FALLBACK).is_some());
+    proc_set_at_secure2(
+        pid,
+        elf_type,
+        *FORCE_PROC,
+        *PROC_MEM_F,
+        *HAVE_CROSS_MEMORY_ATTACH,
+    )
+}
+
+/// Sets the AT_SECURE value to 1 in the auxiliary vector of the
+/// specified process.
+pub fn proc_set_at_secure2(
+    pid: Pid,
+    elf_type: ElfType,
+    force_proc: bool,
+    proc_mem_f: bool,
+    have_cma: bool,
+) -> Result<(), Errno> {
+    // This function requires CONFIG_CROSS_MEMORY_ATTACH,
+    // unless Syd is explicitly configured to fallback.
+    let mut proc_mem = if force_proc || (proc_mem_f && !have_cma) {
+        let mut path = XPathBuf::from_pid(pid);
+        path.push(b"mem");
+
+        let mut flags = OFLAG_PROC;
+        flags.remove(OFlag::O_RDONLY);
+        flags.insert(OFlag::O_RDWR);
+        match safe_open(PROC_FILE(), &path, flags) {
+            Ok(fd) => Some(File::from(fd)),
+            Err(_) => return Err(Errno::EACCES),
+        }
+    } else {
+        None
+    };
+
+    fn proc_read(
+        proc_mem: Option<&mut File>,
+        pid: Pid,
+        buf: &mut [u8],
+        base_addr: usize,
+    ) -> Result<usize, Errno> {
+        match proc_mem {
+            Some(file) => {
+                file.seek(SeekFrom::Start(base_addr as u64))
+                    .map_err(|err| err2no(&err))?;
+
+                let mut nread = 0;
+                while nread < buf.len() {
+                    #[allow(clippy::arithmetic_side_effects)]
+                    match read(&file, &mut buf[nread..]) {
+                        Ok(0) => return Err(Errno::EIO),
+                        Ok(n) => nread += n,
+                        Err(Errno::EINTR) => {}
+                        Err(errno) => return Err(errno),
+                    }
+                }
+
+                Ok(nread)
+            }
+            None => {
+                let local_len = buf.len();
+                let mut local_iov = [IoSliceMut::new(buf)];
+                let remote_iov = [RemoteIoVec {
+                    base: base_addr,
+                    len: local_len,
+                }];
+                process_vm_readv(pid, &mut local_iov, &remote_iov)
+            }
+        }
+    }
+
+    fn proc_write(
+        proc_mem: Option<&mut File>,
+        pid: Pid,
+        buf: &[u8],
+        base_addr: usize,
+    ) -> Result<usize, Errno> {
+        match proc_mem {
+            Some(file) => {
+                file.seek(SeekFrom::Start(base_addr as u64))
+                    .map_err(|err| err2no(&err))?;
+
+                let mut nwrite = 0;
+                while nwrite < buf.len() {
+                    #[allow(clippy::arithmetic_side_effects)]
+                    match write(&file, &buf[nwrite..]) {
+                        Ok(0) => return Err(Errno::EIO),
+                        Ok(n) => nwrite += n,
+                        Err(Errno::EINTR) => {}
+                        Err(errno) => return Err(errno),
+                    }
+                }
+
+                Ok(nwrite)
+            }
+            None => {
+                let local_len = buf.len();
+                let local_iov = [IoSlice::new(buf)];
+                let remote_iov = [RemoteIoVec {
+                    base: base_addr,
+                    len: local_len,
+                }];
+                process_vm_writev(pid, &local_iov, &remote_iov)
+            }
+        }
+    }
+
     // Get the address of the start of the stack for the process.
     let sp = proc_stack_start(pid)?;
     let sp = usize::try_from(sp).or(Err(Errno::EFAULT))?;
@@ -1447,12 +1564,7 @@ pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
 
     // Read argc (the argument count).
     let mut buf = [0u8; 8]; // Max size needed for usize.
-    let mut local_iov = [IoSliceMut::new(&mut buf[..sizeof_ptr])];
-    let remote_iov = [RemoteIoVec {
-        base: sp,
-        len: sizeof_ptr,
-    }];
-    let bytes_read = process_vm_readv(pid, &mut local_iov, &remote_iov)?;
+    let bytes_read = proc_read(proc_mem.as_mut(), pid, &mut buf[..sizeof_ptr], sp)?;
     if bytes_read != sizeof_ptr {
         return Err(Errno::EIO);
     }
@@ -1484,12 +1596,12 @@ pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
     loop {
         // Read one pointer at a time.
         let mut envp_buf = [0u8; 8];
-        let mut local_iov = [IoSliceMut::new(&mut envp_buf[..sizeof_ptr])];
-        let remote_iov = [RemoteIoVec {
-            base: sp.checked_add(offset).ok_or(Errno::EINVAL)?,
-            len: sizeof_ptr,
-        }];
-        let bytes_read = process_vm_readv(pid, &mut local_iov, &remote_iov)?;
+        let bytes_read = proc_read(
+            proc_mem.as_mut(),
+            pid,
+            &mut envp_buf[..sizeof_ptr],
+            sp.checked_add(offset).ok_or(Errno::EINVAL)?,
+        )?;
         if bytes_read != sizeof_ptr {
             return Err(Errno::EIO);
         }
@@ -1508,14 +1620,8 @@ pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
     // fs/exec.c in kernel sources for more information.
     const READ_SIZE: usize = 512; // Read up to 512 bytes.
     let mut buf = [0u8; READ_SIZE];
-    let mut local_iov = [IoSliceMut::new(&mut buf)];
     let sp = sp.checked_add(offset).ok_or(Errno::EINVAL)?;
-    let remote_iov = [RemoteIoVec {
-        base: sp,
-        len: READ_SIZE,
-    }];
-
-    let bytes_read = process_vm_readv(pid, &mut local_iov, &remote_iov)?;
+    let bytes_read = proc_read(proc_mem.as_mut(), pid, &mut buf, sp)?;
     if bytes_read == 0 {
         return Err(Errno::EIO);
     }
@@ -1604,22 +1710,21 @@ pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
                 let val = usize_to_ne_bytes(1, sizeof_ptr);
                 buf[key_end..val_end].copy_from_slice(&val);
 
-                // Prepare to write back the modified value.
-                let local_iov = [IoSlice::new(&buf[key_end..val_end])];
-                let remote_iov = [RemoteIoVec {
-                    base: sp.checked_add(key_end).ok_or(Errno::EINVAL)?,
-                    len: sizeof_ptr,
-                }];
-
                 // SAFETY: We have verified that the auxiliary vector
                 // contains the expected keys in the correct order, and
                 // that the UID/GID values match our own. This ensures
                 // that we are modifying a trusted process. We also
                 // ensure that the offsets are within the bounds of the
                 // read buffer, preventing overflows.
-
+                //
                 // Write the modified value back to the target process.
-                if process_vm_writev(pid, &local_iov, &remote_iov)? != sizeof_ptr {
+                if proc_write(
+                    proc_mem.as_mut(),
+                    pid,
+                    &buf[key_end..val_end],
+                    sp.checked_add(key_end).ok_or(Errno::EINVAL)?,
+                )? != sizeof_ptr
+                {
                     return Err(Errno::EIO);
                 }
 
@@ -1748,26 +1853,11 @@ pub fn log_proc_pid_fd(pid: Option<Pid>) -> Result<(), Errno> {
 
 #[cfg(test)]
 mod tests {
-    use std::{
-        io::Write,
-        os::unix::{fs::PermissionsExt, process::CommandExt},
-        process::Command,
-    };
+    use std::os::unix::fs::PermissionsExt;
 
-    use nix::{
-        sys::{
-            ptrace,
-            ptrace::Options,
-            signal::{kill, Signal},
-            stat::umask,
-            wait::{waitpid, WaitPidFlag, WaitStatus},
-        },
-        unistd::{fork, ForkResult},
-    };
-    use tempfile::tempdir_in;
+    use nix::sys::stat::umask;
 
     use super::*;
-    use crate::{elf::ELFTYPE_NATIVE, xpath};
 
     fn setup() -> bool {
         let _ = crate::log::log_init_simple(crate::syslog::LogLevel::Warn);
@@ -1815,311 +1905,6 @@ mod tests {
         umask(Mode::from_bits_truncate(0o0022));
     }
 
-    #[test]
-    fn test_proc_fd() {
-        let this = Pid::this();
-        let that = Pid::from_raw(1);
-
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{this}/cwd"), true),
-            Ok(Some((libc::AT_FDCWD, this)))
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{this}/exe"), true),
-            Ok(Some((-2, this)))
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{this}/root"), true),
-            Ok(Some((-1, this)))
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/fd"), true),
-            Err(Errno::EACCES)
-        );
-        assert_eq!(proc_fd(this, &xpath!("/proc/{that}/fd"), false), Ok(None));
-        assert_eq!(proc_fd(this, &xpath!("/proc/{this}/fd"), true), Ok(None));
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{this}/fd/0"), true),
-            Ok(Some((0, this)))
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{this}/fd/42"), true),
-            Ok(Some((42, this)))
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{this}/fd/1984"), true),
-            Ok(Some((1984, this)))
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{this}/task/{that}/fd/7"), true),
-            Ok(Some((7, that)))
-        );
-
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/cwd"), true),
-            Err(Errno::EACCES)
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/exe"), true),
-            Err(Errno::EACCES)
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/root"), true),
-            Err(Errno::EACCES)
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/fd/0"), true),
-            Err(Errno::EACCES)
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/task/{this}/fd/7"), true),
-            Err(Errno::EACCES)
-        );
-
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/cwd"), false),
-            Ok(Some((libc::AT_FDCWD, that)))
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/exe"), false),
-            Ok(Some((-2, that)))
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/root"), false),
-            Ok(Some((-1, that)))
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/fd/0"), false),
-            Ok(Some((0, that)))
-        );
-        assert_eq!(
-            proc_fd(this, &xpath!("/proc/{that}/task/{this}/fd/7"), false),
-            Ok(Some((7, this)))
-        );
-    }
-
-    #[test]
-    fn test_proc_set_at_secure_test_native_dynamic() -> Result<(), Errno> {
-        proc_set_at_secure_test(false, false, false)
-    }
-
-    #[test]
-    fn test_proc_set_at_secure_test_native_static() -> Result<(), Errno> {
-        proc_set_at_secure_test(false, true, false)
-    }
-
-    #[test]
-    fn test_proc_set_at_secure_test_native_dynamic_pie() -> Result<(), Errno> {
-        proc_set_at_secure_test(false, false, true)
-    }
-
-    #[test]
-    fn test_proc_set_at_secure_test_native_static_pie() -> Result<(), Errno> {
-        proc_set_at_secure_test(false, true, true)
-    }
-
-    #[test]
-    fn test_proc_set_at_secure_test_32bit_dynamic() -> Result<(), Errno> {
-        proc_set_at_secure_test(true, false, false)
-    }
-
-    #[test]
-    fn test_proc_set_at_secure_test_32bit_static() -> Result<(), Errno> {
-        proc_set_at_secure_test(true, true, false)
-    }
-
-    #[test]
-    fn test_proc_set_at_secure_test_32bit_dynamic_pie() -> Result<(), Errno> {
-        proc_set_at_secure_test(true, false, true)
-    }
-
-    #[test]
-    fn test_proc_set_at_secure_test_32bit_static_pie() -> Result<(), Errno> {
-        proc_set_at_secure_test(true, true, true)
-    }
-
-    fn proc_set_at_secure_test(arch32: bool, statik: bool, pie: bool) -> Result<(), Errno> {
-        if !setup() {
-            return Ok(());
-        }
-
-        // Write the C program.
-        let c_program = r#"
-        #include <stdlib.h>
-        #include <sys/auxv.h>
-
-        int main(void) {
-            return getauxval(AT_SECURE) ? EXIT_SUCCESS : EXIT_FAILURE;
-        }
-        "#;
-
-        // Write the C program to a file in the current directory.
-        let temp_dir = tempdir_in("/tmp").expect("Failed to create temporary directory!");
-        let src_path = temp_dir.path().join("at_secure_test.c");
-        let exe_path = temp_dir.path().join("at_secure_test");
-        let mut src_file = File::create(&src_path).expect("Failed to create C source file!");
-        write!(src_file, "{c_program}").expect("Failed to write C source file!");
-
-        // Compile the C program using "cc" command
-        let mut cmd = Command::new("cc");
-        let elf_type = if arch32 {
-            cmd.arg("-m32");
-            ElfType::Elf32
-        } else {
-            ELFTYPE_NATIVE
-        };
-        if statik && pie {
-            cmd.arg("-static-pie");
-        } else if statik {
-            cmd.arg("-static");
-        }
-        cmd.arg(&src_path).arg("-o").arg(&exe_path);
-        match cmd.output() {
-            Ok(output) => {
-                if !output.status.success() {
-                    eprintln!(
-                        "Compilation failed with arch32={arch32} static:{statik} pie:{pie}: {}",
-                        String::from_utf8_lossy(&output.stderr)
-                    );
-                    return Ok(()); // Skip test if compilation failed.
-                }
-            }
-            Err(e) => {
-                eprintln!("Failed to execute cc command: {e}");
-                return Ok(()); // Skip test if compiler not available.
-            }
-        }
-
-        // Fork and execve the compiled program under ptrace.
-        #[allow(unsafe_code)]
-        match unsafe { fork() } {
-            Ok(ForkResult::Child) => {
-                // Child process
-                if ptrace::traceme().is_err() {
-                    eprintln!("Child: ptrace::traceme failed!");
-                    std::process::exit(1);
-                }
-                let _ = kill(Pid::this(), Signal::SIGSTOP);
-                let _ = Command::new(&exe_path).env("LD_SHOW_AUXV", "1").exec();
-                eprintln!("Child: exec failed!");
-                std::process::exit(127);
-            }
-            Ok(ForkResult::Parent { child }) => {
-                // Parent process
-                match waitpid(child, None) {
-                    Ok(WaitStatus::Stopped(_, Signal::SIGSTOP)) => {
-                        // Child is stopped, proceed...
-                    }
-                    Ok(status) => {
-                        eprintln!("Parent: Unexpected wait status: {status:?}");
-                        return Err(Errno::EIO);
-                    }
-                    Err(e) => {
-                        eprintln!("Parent: waitpid failed: {e}");
-                        return Err(Errno::EIO);
-                    }
-                }
-
-                // Set ptrace options to get PTRACE_EVENT_EXEC.
-                // Set exit-kill to ensure no stray child processes.
-                if let Err(e) = ptrace::setoptions(
-                    child,
-                    Options::PTRACE_O_TRACEEXEC | Options::PTRACE_O_EXITKILL,
-                ) {
-                    eprintln!("Parent: ptrace::setoptions failed: {e}");
-                    return Err(e);
-                }
-
-                // Continue the child
-                if let Err(e) = ptrace::cont(child, None) {
-                    eprintln!("Parent: ptrace::cont failed: {e}");
-                    return Err(e);
-                }
-
-                // Wait for PTRACE_EVENT_EXEC
-                loop {
-                    match waitpid(child, Some(WaitPidFlag::empty())) {
-                        Ok(WaitStatus::PtraceEvent(_, Signal::SIGTRAP, event))
-                            if event == ptrace::Event::PTRACE_EVENT_EXEC as i32 =>
-                        {
-                            break;
-                        }
-                        Ok(WaitStatus::Stopped(_, _)) => {
-                            if let Err(e) = ptrace::cont(child, None) {
-                                eprintln!("Parent: ptrace::cont failed during loop: {e}");
-                                return Err(e);
-                            }
-                        }
-                        Ok(WaitStatus::Exited(_, status)) => {
-                            eprintln!("Child exited unexpectedly with status {status}");
-                            return Err(Errno::EIO);
-                        }
-                        Ok(WaitStatus::Signaled(_, sig, _)) => {
-                            eprintln!("Child terminated by signal {sig:?}");
-                            return Err(Errno::EIO);
-                        }
-                        Ok(status) => {
-                            eprintln!("Parent: Unexpected wait status: {status:?}");
-                        }
-                        Err(e) => {
-                            eprintln!("Parent: waitpid failed: {e}");
-                            return Err(Errno::EIO);
-                        }
-                    }
-                }
-
-                // Call our function to set AT_SECURE.
-                if let Err(e) = proc_set_at_secure(child, elf_type) {
-                    eprintln!("proc_set_at_secure failed: {e}");
-                    return Err(e);
-                }
-
-                // Continue the child
-                if let Err(e) = ptrace::cont(child, None) {
-                    eprintln!("Parent: ptrace::cont failed after setting AT_SECURE: {e}");
-                    return Err(e);
-                }
-
-                // Wait for the child to exit
-                loop {
-                    match waitpid(child, None) {
-                        Ok(WaitStatus::Exited(_, status_code)) => {
-                            if status_code != 0 {
-                                eprintln!("Child exited with failure status {status_code}");
-                                return Err(Errno::EIO);
-                            }
-
-                            // Test passed!
-                            return Ok(());
-                        }
-                        Ok(WaitStatus::Signaled(_, sig, _)) => {
-                            eprintln!("Child terminated by signal {sig:?}");
-                            return Err(Errno::EIO);
-                        }
-                        Ok(WaitStatus::Stopped(_, _)) => {
-                            // Continue the child.
-                            if let Err(e) = ptrace::cont(child, None) {
-                                eprintln!("Parent: ptrace::cont failed during final loop: {e}");
-                                return Err(e);
-                            }
-                        }
-                        Ok(status) => {
-                            eprintln!("Parent: Unexpected wait status: {status:?}");
-                        }
-                        Err(e) => {
-                            eprintln!("Parent: waitpid failed: {e}");
-                            return Err(Errno::EIO);
-                        }
-                    }
-                }
-            }
-            Err(e) => {
-                eprintln!("Fork failed: {e}");
-                return Err(Errno::EIO);
-            }
-        }
-    }
-
     // Skip test if PROCMAP_QUERY ioctl(2) is not supported.
     macro_rules! skip_if_procmap_query_not_supported {
         () => {
@@ -2446,16 +2231,6 @@ mod tests {
         );
     }
 
-    #[test]
-    fn test_proc_executables_invalid_pid() {
-        if !setup() {
-            return;
-        }
-
-        // PID -1 should fail to open /proc/-1/maps.
-        assert_eq!(proc_executables(Pid::from_raw(-1)), Err(Errno::ENOENT));
-    }
-
     #[test]
     fn test_proc_executables_ok_non_empty() {
         if !setup() {
