@@ -271,6 +271,18 @@ macro_rules! assert_status_not_supported {
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_operation_not_supported {
+    ($status:expr) => {
+        let code = $status.code().unwrap_or(127);
+        assert!(
+            matches!(code, libc::EOPNOTSUPP),
+            "code:{code} status:{:?}",
+            $status
+        );
+    };
+}
+
 #[macro_export]
 macro_rules! skip_unless_available {
     ($($program:expr),* $(,)?) => {{
@@ -300,6 +312,22 @@ macro_rules! skip_if_32bin_64host {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_if_mips {
+    () => {{
+        if cfg!(any(
+            target_arch = "mips",
+            target_arch = "mips32r6",
+            target_arch = "mips64",
+            target_arch = "mips64r6",
+        )) {
+            eprintln!("Test does not work on MIPS, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_if_root {
     () => {{
@@ -313,6 +341,27 @@ macro_rules! skip_if_root {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_unless_linux {
+    ($need_major:expr, $need_minor:expr) => {{
+        let (maj, min) = *syd::config::KERNEL_VERSION;
+        let too_old = maj < $need_major || (maj == $need_major && min < $need_minor);
+        if too_old {
+            eprintln!(
+                "Test requires Linux kernel >= {}.{} (current: {maj}.{min}), skipping!",
+                $need_major, $need_minor,
+            );
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        } else {
+            eprintln!(
+                "Linux kernel {maj}.{min} satisfies >= {}.{}, proceeding!",
+                $need_major, $need_minor,
+            );
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_stdin_is_a_tty {
     () => {{
@@ -326,11 +375,24 @@ macro_rules! skip_unless_stdin_is_a_tty {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_unless_stdout_is_a_tty {
+    () => {{
+        if !nix::unistd::isatty(std::io::stdout()).unwrap_or(false) {
+            eprintln!("Test requires STDOUT to be a TTY, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        } else {
+            eprintln!("STDOUT is a TTY, proceeding with test...");
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_kernel_crypto_is_supported {
     () => {{
         let key = syd::hash::Key::random().unwrap();
-        match syd::hash::aes_ctr_setup(&key) {
+        match syd::hash::aes_ctr_setup(key.as_ref()) {
             Ok(fd) => drop(fd),
             Err(nix::errno::Errno::EAFNOSUPPORT) => {
                 eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
@@ -380,16 +442,39 @@ macro_rules! skip_unless_xattrs_are_supported {
 
 #[macro_export]
 macro_rules! skip_unless_unshare {
-    () => {{
-        match check_unshare() {
-            Some(false) => {
-                eprintln!("Test requires Linux namespaces, skipping!");
+    ( $( $ns:expr ),* $(,)? ) => {{
+        use nix::sched::CloneFlags;
+
+        // Build combined CloneFlags from the provided string literals.
+        let mut flags = CloneFlags::empty();
+        $(
+            flags |= match $ns {
+                "mount"   => CloneFlags::CLONE_NEWNS,
+                "uts"     => CloneFlags::CLONE_NEWUTS,
+                "ipc"     => CloneFlags::CLONE_NEWIPC,
+                "user"    => CloneFlags::CLONE_NEWUSER,
+                "pid"     => CloneFlags::CLONE_NEWPID,
+                "net"     => CloneFlags::CLONE_NEWNET,
+                "cgroup"  => CloneFlags::CLONE_NEWCGROUP,
+                "time"    => syd::confine::CLONE_NEWTIME,
+                "all"     => CloneFlags::CLONE_NEWNS |  CloneFlags::CLONE_NEWUTS | CloneFlags::CLONE_NEWIPC | CloneFlags::CLONE_NEWUSER | CloneFlags::CLONE_NEWPID | CloneFlags::CLONE_NEWNET | CloneFlags::CLONE_NEWCGROUP | syd::confine::CLONE_NEWTIME,
+                other => panic!("Unknown namespace: {other}"),
+            };
+        )*
+
+        if flags.is_empty() {
+            panic!("No namespace given!");
+        }
+
+        match syd::confine::ns_enabled(flags) {
+            Ok(false) => {
+                eprintln!("Test requires Linux namespaces ({:?}), skipping!", [$($ns),*]);
                 std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
                 return Ok(());
             }
-            None => {
+            Err(errno) => {
                 return Err(TestError(
-                    "Failed to test for Linux namespaces!".to_string(),
+                    format!("Failed to test for Linux namespaces: {errno}"),
                 ));
             }
             _ => {}
@@ -490,6 +575,17 @@ macro_rules! skip_unless_landlock_abi_supported {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_if_cross_memory_attach_is_not_enabled {
+    () => {
+        if !*syd::config::HAVE_CROSS_MEMORY_ATTACH {
+            eprintln!("skipping: kernel not configured with CONFIG_CROSS_MEMORY_ATTACH");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+    };
+}
+
 #[macro_export]
 macro_rules! skip_if_strace {
     () => {{
@@ -501,6 +597,17 @@ macro_rules! skip_if_strace {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_unless_strace_can_inject {
+    () => {{
+        if !check_strace_inject() {
+            eprintln!("strace can't inject syscalls, cannot run test, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_pty {
     () => {{
@@ -608,6 +715,11 @@ pub static SYD_INFO: Lazy<String> =
 pub static SYD_LOCK: Lazy<String> =
     Lazy::new(|| env::var("CARGO_BIN_EXE_syd-lock").unwrap_or("syd-lock".to_string()));
 
+/// Holds a `String` to run `syd-mdwe`.
+/// Honours CARGO_BIN_EXE_syd-mdwe environment variable.
+pub static SYD_MDWE: Lazy<String> =
+    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-mdwe").unwrap_or("syd-mdwe".to_string()));
+
 /// Holds a `String` to run `syd-pds`.
 /// Honours CARGO_BIN_EXE_syd-pds environment variable.
 pub static SYD_PDS: Lazy<String> =
@@ -815,7 +927,10 @@ pub fn syd() -> Syd {
         cmd.arg("--");
     } else if *USE_STRACE {
         cmd.arg("strace");
-        cmd.arg("-f");
+
+        if env::var_os("SYD_TEST_STRACE_NOFORK").is_none() {
+            cmd.arg("-f");
+        }
 
         if env::var_os("SYD_TEST_STRACE_VERBOSE").is_none() {
             cmd.arg("-s256");
@@ -861,7 +976,7 @@ pub fn syd() -> Syd {
     // Allow coredumps and non-PIE.
     cmd.m("trace/allow_unsafe_prlimit:1"); // Allow coredumps.
     cmd.m("trace/allow_unsafe_nopie:1"); // Allow non-PIE.
-    if *USE_STRACE || *USE_TRACE {
+    if *USE_TRACE || (*USE_STRACE && env::var_os("SYD_TEST_STRACE_NOFORK").is_none()) {
         cmd.m("trace/allow_unsafe_ptrace:1"); // Disable ptrace.
     }
     cmd
@@ -915,17 +1030,6 @@ pub fn check_ipv6() -> bool {
     }
 }
 
-/// Check if namespaces are supported.
-/// Returns None if syd process was terminated by a signal.
-pub fn check_unshare() -> Option<bool> {
-    syd()
-        .args(["-poff", "-pD", "-pcontainer", "true"])
-        .status()
-        .map(|stat| stat.code())
-        .ok()?
-        .map(|code| code == 0)
-}
-
 /// Checks if the C compiler allows nested routines.
 /// Returns `true` if supported, `false` otherwise.
 /// If successful, creates the executable `nested` in CWD.
@@ -1190,6 +1294,26 @@ pub fn check_32bin_64host() -> bool {
     }
 }
 
+/// Checks whether strace supports syscall injection.
+pub fn check_strace_inject() -> bool {
+    let status = Command::new("strace")
+        .arg("-qq")
+        .arg("-e")
+        .arg("trace=getpid")
+        .arg("-e")
+        .arg("inject=getpid:retval=2525")
+        .arg("--")
+        .arg("sh")
+        .arg("-c")
+        .arg("test $$ -eq 2525")
+        .status();
+
+    match status {
+        Ok(status) => status.success(),
+        Err(_) => false,
+    }
+}
+
 /// Enable coredumps.
 pub fn enable_coredumps() -> Result<(), Errno> {
     // Set both the soft and hard limits
