@@ -1475,7 +1475,7 @@ struct Opts {
     subcmd: SubCommand,
 }
 
-fn main() -> SydResult<ExitCode> {
+syd::main! {
     // Guard against CVE-2019-5736:
     // Copy /proc/self/exe in an anonymous fd (created via memfd_create), seal it and re-execute it.
     // See:
@@ -1495,7 +1495,6 @@ fn main() -> SydResult<ExitCode> {
     let mut opts = Opts::parse();
 
     // Initialize sandbox logging and Youki tracing.
-    env::set_var(syd::config::ENV_NO_SYSLOG, "NoThanks");
     let (level, trace_level) = if opts.global.debug {
         (LogLevel::Debug, tracing::Level::DEBUG)
     } else {
