@@ -10,7 +10,6 @@
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
-    collections::{HashMap, HashSet},
     env,
     io::stderr,
     net::{IpAddr, Ipv4Addr, SocketAddr},
@@ -32,6 +31,7 @@ use nix::{
     sys::{
         epoll::{Epoll, EpollCreateFlags, EpollEvent, EpollFlags},
         mman::{mmap_anonymous, MapFlags, ProtFlags},
+        prctl::set_dumpable,
         resource::{getrlimit, setrlimit, Resource},
         socket::{
             accept4, connect, listen, shutdown, socket, AddressFamily, Backlog, Shutdown, SockFlag,
@@ -39,29 +39,29 @@ use nix::{
         },
         stat::Mode,
     },
-    unistd::{chdir, chroot, close, pipe2, sysconf, write, SysconfVar},
+    unistd::{chdir, chroot, pipe2, sysconf, write, SysconfVar},
 };
 use sendfd::RecvWithFd;
 use syd::{
     compat::epoll_ctl_safe,
     config::PIPE_BUF,
+    confine::{confine_mdwe, seccomp_native_has_socketcall},
     err::{SydError, SydResult},
-    fs::{duprand, fillrandom, randint, retry_on_eintr},
-    hash::SydRandomState,
+    fs::{closeexcept, duprand, fillrandom, randint, retry_on_eintr},
+    hash::SydHashMap,
     landlock_policy::LandlockPolicy,
+    parsers::sandbox::PortSet,
     path::XPathBuf,
     sealbox::{
         check_madvise_guard_support, madvise_guard_install, mprotect_none, mprotect_readonly, mseal,
     },
-    seccomp_native_has_socketcall,
 };
 
 // Pong reply to Syd for debugging.
 const PONG: &[u8] =
     b"{\"ctx\":\"recv_proxy_fd\",\"tor\":\"pong\",\"msg\":\"Action brings good fortune.\"}\n\0";
 
-#[allow(clippy::disallowed_methods)]
-fn main() -> SydResult<ExitCode> {
+syd::main! {
     // Parse options.
     let mut opts = parse_options()?;
 
@@ -189,16 +189,16 @@ fn main() -> SydResult<ExitCode> {
     };
 
     // Set mapping back to read-only.
-    mprotect_readonly(addr_ptr, page_siz.get())?;
+    mprotect_readonly(addr_ptr, page_siz)?;
 
     // Set guard-pages to PROT_NONE.
-    mprotect_none(guard_ptr_1, page_siz.get())?;
-    mprotect_none(guard_ptr_2, page_siz.get())?;
+    mprotect_none(guard_ptr_1, page_siz)?;
+    mprotect_none(guard_ptr_2, page_siz)?;
 
     // Install lightweight guard-pages if supported.
     if check_madvise_guard_support() {
-        madvise_guard_install(guard_ptr_1, page_siz.get())?;
-        madvise_guard_install(guard_ptr_2, page_siz.get())?;
+        madvise_guard_install(guard_ptr_1, page_siz)?;
+        madvise_guard_install(guard_ptr_2, page_siz)?;
     }
 
     // Protect the memory area of address pointer,
@@ -206,7 +206,7 @@ fn main() -> SydResult<ExitCode> {
     //
     // ENOSYS: mseal(2) is not implemented (Linux>=6.10)
     // EPERM: Sealing is supported only on 64-bit CPUs, 32-bit is not supported.
-    match mseal(base_ptr, base_siz.get()) {
+    match mseal(base_ptr, base_siz) {
         Ok(_) | Err(Errno::ENOSYS | Errno::EPERM) => {}
         Err(errno) => return Err(errno.into()),
     }
@@ -230,8 +230,8 @@ fn main() -> SydResult<ExitCode> {
     })
 }
 
-type FdMap = HashMap<RawFd, RawFd, SydRandomState>;
-type CliMap = HashMap<RawFd, Client, SydRandomState>;
+type FdMap = SydHashMap<RawFd, RawFd>;
+type CliMap = SydHashMap<RawFd, Client>;
 
 struct Proxy<'a> {
     debug: bool,
@@ -270,7 +270,7 @@ impl<'a> Proxy<'a> {
             let _ = write(stderr(), PONG);
         }
 
-        // TODO: MAX_EVENTS=1024 move to config.rs
+        // TODO: TOR_EPOLL_MAX_EVENTS=1024 move to config.rs
         let mut events = vec![EpollEvent::empty(); 1024];
         loop {
             // Wait for events and handle EINTR.
@@ -350,7 +350,7 @@ impl<'a> Proxy<'a> {
             "exit",
             "exit_group",
             // can {{dr}e,}allocate memory.
-            // mmap{,2} and mprotect are further confined.
+            // mmap{,2} and mprotect are further confined to disable PROT_EXEC.
             "brk",
             "madvise",
             "mremap",
@@ -488,8 +488,8 @@ impl<'a> Proxy<'a> {
             // Ignore errors as Landlock may not be supported.
             let abi = syd::landlock::ABI::new_current();
             let conn_portset = port.map(|port| {
-                let mut set = HashSet::default();
-                set.insert(port..=port);
+                let mut set = PortSet::with_capacity(0x10000);
+                set.insert(port.into());
                 set
             });
             let policy = LandlockPolicy {
@@ -504,14 +504,10 @@ impl<'a> Proxy<'a> {
 
             // Set up Memory-Deny-Write-Execute protections.
             // Ignore errors as PR_SET_MDWE may not be supported.
-            const PR_SET_MDWE: libc::c_int = 65;
-            const PR_MDWE_REFUSE_EXEC_GAIN: libc::c_ulong = 1;
-            // SAFETY: nix has no wrapper for PR_SET_MDWE.
-            let _ = unsafe { libc::prctl(PR_SET_MDWE, PR_MDWE_REFUSE_EXEC_GAIN, 0, 0, 0) };
+            let _ = confine_mdwe(false);
 
             // Set the process dumpable attribute to not-dumpable.
-            // SAFETY: Our nix version does not have a wrapper for set_dumpable yet.
-            let _ = unsafe { libc::prctl(libc::PR_SET_DUMPABLE, 0, 0, 0, 0) };
+            let _ = set_dumpable(false);
 
             // Deny reading the timestamp counter (x86 only).
             // SAFETY: Our nix version does not have a wrapper for SET_TSC yet.
@@ -569,7 +565,7 @@ impl<'a> Proxy<'a> {
             let _ = client.handle_input(is_ext);
         }
 
-        if is_out {
+        if is_out || is_hup || is_err {
             // Handle writable events.
             let _ = client.handle_output(is_ext);
         }
@@ -949,19 +945,18 @@ fn parse_options() -> SydResult<TorOpts> {
     // Close all file descriptors, except:
     // 1. Standard input, output, and error which are set to /dev/null by Syd.
     // 2. The PID fd and the socket fd passed by the Syd process.
-    // We do it early here so FD randomization does not reduce performance.
-    let max_fd = std::cmp::max(fpid, sock);
-    for fd in 3..=max_fd {
-        if fd == fpid || fd == sock {
-            continue;
-        }
-
-        // SAFETY: Nothing can be done on close errors.
-        let _ = close(fd);
+    // Nothing can be done on closeexcept errors.
+    // We do it early here so FD randomization doesn't effect performance.
+    #[allow(clippy::cast_sign_loss)]
+    {
+        let fd1 = fpid as libc::c_uint;
+        let fd2 = sock as libc::c_uint;
+        let _ = closeexcept(&if fd1 < fd2 {
+            [0, 1, 2, fd1, fd2]
+        } else {
+            [0, 1, 2, fd2, fd1]
+        });
     }
-    // SAFETY: nix does not have a close_range wrapper yet.
-    // Nothing can be done on close_range errors.
-    let _ = unsafe { libc::syscall(libc::SYS_close_range, max_fd + 1, RawFd::MAX, 0) };
 
     // Validate socket address and convert.
     let mut unix_fd = None;
