@@ -245,6 +245,22 @@ fn dowild<'a>(p: &[u8], mut text: &'a [u8], mut a: &'a [&'a [u8]]) -> MatchResul
 
                 let mut next_start = 0;
                 while next_start <= text.len() {
+                    if is_double_star
+                        && p_idx >= 4
+                        && p[p_idx - 4] == b'/'
+                        && p[p_idx - 3] == b'*'
+                        && p[p_idx - 2] == b'*'
+                        && p[p_idx - 1] == b'/'
+                        && next_start > 0
+                        && text[next_start - 1] != b'/'
+                    {
+                        // Ensure component-anchored matching after "/**/".
+                        // Prevent mid-component matches (e.g., /usr/**/bin !~ /usr/sabin)
+                        // and avoid drifting ".*/" into names (e.g., / ** /.*/ ** !~ /a/b.c/...).
+                        // Zero-segment behavior is preserved (e.g., /**/bin matches /bin).
+                        next_start += 1;
+                        continue;
+                    }
                     if next_start == text.len() {
                         // Attempt to move to the next segment if available.
                         if let Some(next_text) = a.first() {
